!  ==============================================================================================================================\n
!  MODULE 	: sechiba
! 
!  CONTACT     : orchidee-help _at_ ipsl.jussieu.fr
!
!  LICENCE      : IPSL (2006)
!  This software is governed by the CeCILL licence see ORCHIDEE/ORCHIDEE_CeCILL.LIC
!
!>\BRIEF        Structures the calculation of atmospheric and hydrological 
!! variables by calling diffuco_main, enerbil_main, hydrolc_main (or hydrol_main),
!! enerbil_fusion, condveg_main and thermosoil_main. Note that sechiba_main
!! calls slowproc_main and thus indirectly calculates the biogeochemical
!! processes as well.
!!
!!\n DESCRIPTION  : :: shumdiag, :: litterhumdiag and :: stempdiag are not 
!! saved in the restart file because at the first time step because they 
!! are recalculated. However, they must be saved as they are in slowproc 
!! which is called before the modules which calculate them.
!! 
!! RECENT CHANGE(S): None 
!! 
!! REFERENCE(S) : None
!!   
!! SVN     :
!! $HeadURL$ 
!! $Date$
!! $Revision$
!! \n
!_ ================================================================================================================================
 
MODULE sechiba
 
  USE ioipsl
  USE xios_orchidee
  
  ! modules used :
  USE constantes
  USE constantes_soil
  USE pft_parameters
  USE structures
  USE diffuco
  USE condveg
  USE enerbil
  USE hydrol     
  USE hydrolc
  USE sechiba_hydrol_arch
  USE thermosoil
  USE sechiba_io
  USE slowproc
  USE routing
  USE ioipsl_para
  USE stomate_laieff
!+++ TEMP ++++
  USE function_library
!+++++++


  IMPLICIT NONE

  ! Private and public routines

  PRIVATE
  PUBLIC sechiba_main,sechiba_clear

  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexveg       !! indexing array for the 3D fields of vegetation
!$OMP THREADPRIVATE(indexveg)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexlai       !! indexing array for the 3D fields of vegetation
!$OMP THREADPRIVATE(indexlai)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexnobio     !! indexing array for the 3D fields of other surfaces (ice,
                                                                     !! lakes, ...)
!$OMP THREADPRIVATE(indexnobio)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexsoil      !! indexing array for the 3D fields of soil types (kjpindex*nstm)
!$OMP THREADPRIVATE(indexsoil)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexgrnd      !! indexing array for the 3D ground heat profiles (kjpindex*ngrnd)
!$OMP THREADPRIVATE(indexgrnd)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexlayer     !! indexing array for the 3D fields of soil layers in CWRR (kjpindex*nslm)
!$OMP THREADPRIVATE(indexlayer)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexnbdl      !! indexing array for the 3D fields of diagnostic soil layers (kjpindex*nbdl) 
!$OMP THREADPRIVATE(indexnbdl)  
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexalb       !! indexing array for the 2 fields of albedo
!$OMP THREADPRIVATE(indexalb)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexsnow      !! indexing array for the 3D fields snow layers 
!$OMP THREADPRIVATE(indexsnow)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: indexcan       !! indexing array for the level fields of the canopy
!$OMP THREADPRIVATE(indexcan)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: jnlevels_loc    !! (JR) number of vegetation levels (integer)
!$OMP THREADPRIVATE(jnlevels_loc)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: veget          !! Fraction of vegetation type (unitless, 0-1)       
!$OMP THREADPRIVATE(veget)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: veget_max      !! Max. fraction of vegetation type (LAI -> infty, unitless)
!$OMP THREADPRIVATE(veget_max)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: height         !! Vegetation Height (m)
!$OMP THREADPRIVATE(height)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: totfrac_nobio  !! Total fraction of continental ice+lakes+cities+...
                                                                     !! (unitless, 0-1)
!$OMP THREADPRIVATE(totfrac_nobio)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: floodout       !! Flow out of floodplains from hydrol
!$OMP THREADPRIVATE(floodout)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: runoff         !! Surface runoff calculated by hydrol or hydrolc 
                                                                     !! @tex $(kg m^{-2})$ @endtex
!$OMP THREADPRIVATE(runoff)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: drainage       !! Deep drainage calculatedd by hydrol or hydrolc 
                                                                     !! @tex $(kg m^{-2})$ @endtex
!$OMP THREADPRIVATE(drainage)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: returnflow     !! Water flow from lakes and swamps which returns to 
                                                                     !! the grid box @tex $(kg m^{-2})$ @endtex
!$OMP THREADPRIVATE(returnflow)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: reinfiltration !! Routed water which returns into the soil
!$OMP THREADPRIVATE(reinfiltration)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: irrigation     !! Irrigation flux taken from the routing reservoirs and 
                                                                     !! being put into the upper layers of the soil 
                                                                     !! @tex $(kg m^{-2})$ @endtex
!$OMP THREADPRIVATE(irrigation)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: emis           !! Surface emissivity (unitless)
!$OMP THREADPRIVATE(emis)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: z0             !! Surface roughness (m)
!$OMP THREADPRIVATE(z0)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: z0_veg         !! Surface roughness of vegetated part (m) 
!$OMP THREADPRIVATE(z0_veg)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: roughheight    !! Effective height for roughness (m)
!$OMP THREADPRIVATE(roughheight)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: reinf_slope    !! slope coefficient (reinfiltration) 
!$OMP THREADPRIVATE(reinf_slope)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: shumdiag       !! Mean relative soil moisture in the different levels used 
                                                                     !! by thermosoil.f90 (unitless, 0-1)
!$OMP THREADPRIVATE(shumdiag)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: shumdiag_perma !! Saturation degree of the soil  
!$OMP THREADPRIVATE(shumdiag_perma) 
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: k_litt         !! litter cond.
!$OMP THREADPRIVATE(k_litt)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: litterhumdiag  !! Litter dryness factor (unitless, 0-1)
!$OMP THREADPRIVATE(litterhumdiag)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: stempdiag      !! Temperature which controls canopy evolution (K)
!$OMP THREADPRIVATE(stempdiag)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: qsintveg       !! Water on vegetation due to interception 
                                                                     !! @tex $(kg m^{-2})$ @endtex
!$OMP THREADPRIVATE(qsintveg)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vbeta2         !! Interception resistance (unitless,0-1)
!$OMP THREADPRIVATE(vbeta2)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vbeta3         !! Vegetation resistance (unitless,0-1)
!$OMP THREADPRIVATE(vbeta3)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vbeta3pot      !! Potential vegetation resistance
!$OMP THREADPRIVATE(vbeta3pot)
!!$ REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vbetaco2       !! STOMATE:Vegetation resistance to CO2 (unitless,0-1)
!!$!$OMP THREADPRIVATE(vbetaco2)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: gsmean         !! Mean stomatal conductance for CO2 (umol m-2 s-1)
!$OMP THREADPRIVATE(gsmean)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: cimean         !! STOMATE: mean intercellular CO2 concentration (ppm)
!$OMP THREADPRIVATE(cimean)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vevapwet       !! Interception loss over each PFT 
                                                                     !! @tex $(kg m^{-2} days^{-1})$ @endtex
!$OMP THREADPRIVATE(vevapwet)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: transpir       !! Transpiration @tex $(kg m^{-2} days^{-1})$ @endtex
!$OMP THREADPRIVATE(transpir)
   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: transpir_supply  !! Supply of water for transpiration @tex$$(mm dt^{-1})$ @endtex
!$OMP THREADPRIVATE(transpir_supply)
   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vir_transpir_supply !! Supply of water for transpiration @tex$$(mm dt^{-1})$ @endtex
!$OMP THREADPRIVATE(vir_transpir_supply)
   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: stressed      !! Adjusted ecosystem functioning. Takes the unit of the variable
                                                                     !! used as a proxy for waterstress
!$OMP THREADPRIVATE(stressed)
   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: unstressed    !! Initial ecosystem functioning after the first calculation and 
                                                                     !! before any recalculations. Takes the unit of the variable used
                                                                     !! as a proxy for unstressed.
!$OMP THREADPRIVATE(unstressed)
!!$   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vir_transpir_mod !! potential transpiration (transpot) divided by veget_max
!!$!$OMP THREADPRIVATE(vir_transpir_mod)
   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: transpir_mod      !! transpir divided by veget_max
!$OMP THREADPRIVATE(transpir_mod)

   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:, :, :)    :: transpir_column            !! Supply of water for transpiration 
!$OMP THREADPRIVATE(transpir_column)
   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:, :, :)    :: transpir_supply_column     !! Supply of water for transpiration 
!$OMP THREADPRIVATE(transpir_supply_column)
   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:, :, :)    :: transpir_mod_column        !! Supply of water for transpiration 
!$OMP THREADPRIVATE(transpir_mod_column)


   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: transpot      !! Potential transpiration (needed for irrigation)
!$OMP THREADPRIVATE(transpot)
   REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: qsintmax      !! Maximum amount of water in the canopy interception 
                                                                     !! reservoir @tex $(kg m^{-2})$ @endtex
!$OMP THREADPRIVATE(qsintmax)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: rveget         !! Surface resistance for the vegetation 
                                                                     !! @tex $(s m^{-1})$ @endtex
!$OMP THREADPRIVATE(rveget)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: rstruct        !! Vegetation structural resistance
!$OMP THREADPRIVATE(rstruct)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: warnings     !! Holds a count of how many warnings we run into
                                                                     !! of different types.  It will get reset at the
                                                                     !! end of each period since we don't wish to restart it.
                                                                     !! Purely a technical diagnostic variable.
!$OMP THREADPRIVATE(warnings)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: snow_nobio     !! Snow mass of non-vegetative surfaces 
                                                                     !! @tex $(kg m^{-2})$ @endtex
!$OMP THREADPRIVATE(snow_nobio)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: snow_nobio_age !! Snow age on non-vegetative surfaces (days)
!$OMP THREADPRIVATE(snow_nobio_age)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: frac_nobio     !! Fraction of non-vegetative surfaces (continental ice, 
                                                                     !! lakes, ...) (unitless, 0-1)
!$OMP THREADPRIVATE(frac_nobio)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: albedo         !! Surface albedo for visible and near-infrared 
                                                                     !! (unitless, 0-1)
!$OMP THREADPRIVATE(albedo)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:) &
                                                   :: albedo_pft     !! Albedo (two stream radiation transfer model)
                                                                     !! for visible and near-infrared range 
                                                                     !! for each PFT (unitless)
!$OMP THREADPRIVATE(albedo_pft)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:):: assim_param    !! min+max+opt temps, vcmax, vjmax for photosynthesis
!$OMP THREADPRIVATE(assim_param)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: lai            !! Surface foliaire
!$OMP THREADPRIVATE(lai)
  TYPE(laieff_type), ALLOCATABLE, SAVE, DIMENSION (:,:,:)  :: laieff_fit   !! The parameters for fitting the effective
                                                                     !! LAI function
!$OMP THREADPRIVATE(laieff_fit)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: gpp            !! STOMATE: GPP. gC/m**2 of total area
!$OMP THREADPRIVATE(gpp)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)     :: temp_growth    !! Growth temperature (°C) - Is equal to t2m_month
!$OMP THREADPRIVATE(temp_growth)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: humrel         !! Relative humidity
!$OMP THREADPRIVATE(humrel)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: vegstress      !! Vegetation moisture stress (only for vegetation growth)
!$OMP THREADPRIVATE(vegstress)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:):: frac_age       !! Age efficacity from STOMATE for isoprene 
!$OMP THREADPRIVATE(frac_age)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: soiltile       !! Fraction of each soil tile (0-1, unitless)
!$OMP THREADPRIVATE(soiltile)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:) :: njsc           !! Index of the dominant soil textural class in the grid cell (1-nscm, unitless)
!$OMP THREADPRIVATE(njsc)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: vbeta1         !! Snow resistance 
!$OMP THREADPRIVATE(vbeta1)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: vbeta4         !! Bare soil resistance
!$OMP THREADPRIVATE(vbeta4)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: vbeta5         !! Floodplains resistance
!$OMP THREADPRIVATE(vbeta5)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: soilcap        !!
!$OMP THREADPRIVATE(soilcap)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: soilflx        !!
!$OMP THREADPRIVATE(soilflx)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: temp_sol       !! Soil temperature
!$OMP THREADPRIVATE(temp_sol)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: qsurf          !! near soil air moisture
!$OMP THREADPRIVATE(qsurf)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: flood_res      !! flood reservoir estimate
!$OMP THREADPRIVATE(flood_res)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: flood_frac     !! flooded fraction
!$OMP THREADPRIVATE(flood_frac)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: snow           !! Snow mass [Kg/m^2]
!$OMP THREADPRIVATE(snow)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: snow_age       !! Snow age
!$OMP THREADPRIVATE(snow_age)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: drysoil_frac   !! Fraction of visibly (albedo) Dry soil (Between 0 and 1)
!$OMP THREADPRIVATE(drysoil_frac)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: rsol           !! resistance to bare soil evaporation
!$OMP THREADPRIVATE(rsol)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: evap_bare_lim  !! Bare soil stress
!$OMP THREADPRIVATE(evap_bare_lim)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:) ::swc            !! Soil water content (a copy of mc) m3 m-3
!$OMP THREADPRIVATE(swc)
  INTEGER(i_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:) ::veg_soiltext  !! Percentage of each veg. type on each soil type
!$OMP THREADPRIVATE(veg_soiltext)                                      !! of each grid point (copy of corr_veg_soil)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: co2_flux       !! CO2 flux (gC/m**2 of average ground/s)
!$OMP THREADPRIVATE(co2_flux)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: evapot         !! Soil Potential Evaporation
!$OMP THREADPRIVATE(evapot)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: evapot_corr    !! Soil Potential Evaporation Correction (Milly 1992)
!$OMP THREADPRIVATE(evapot_corr)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: vevapflo       !! Floodplains evaporation
!$OMP THREADPRIVATE(vevapflo)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: vevapsno       !! Snow evaporation
!$OMP THREADPRIVATE(vevapsno)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: vevapnu        !! Bare soil evaporation
!$OMP THREADPRIVATE(vevapnu)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: t2mdiag        !! 2 meter temperature
!$OMP THREADPRIVATE(t2mdiag)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: tot_melt       !! Total melt
!$OMP THREADPRIVATE(tot_melt)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: vbeta          !! Resistance coefficient
!$OMP THREADPRIVATE(vbeta)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: valpha         !! Resistance coefficient
!$OMP THREADPRIVATE(valpha)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: fusion         !!
!$OMP THREADPRIVATE(fusion)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: rau            !! Density
!$OMP THREADPRIVATE(rau)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: deadleaf_cover !! Fraction of soil covered by dead leaves
!$OMP THREADPRIVATE(deadleaf_cover)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: ptnlev1        !! 1st level Different levels soil temperature
!$OMP THREADPRIVATE(ptnlev1)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)     :: lai_split      !! TEMPORARY
!$OMP THREADPRIVATE(lai_split)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:) :: Isotrop_Abs_Tot_p  !! Absorbed radiation per level for photosynthesis
!$OMP THREADPRIVATE(Isotrop_Abs_Tot_p)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:) :: Isotrop_Tran_Tot_p  !! Transmitted radiation per level for photosynthesis
!$OMP THREADPRIVATE(Isotrop_Tran_Tot_p)


 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: flux_rn_grid 
!$OMP THREADPRIVATE(flux_rn_grid)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: flux_h_grid 
!$OMP THREADPRIVATE(flux_h_grid)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: flux_le_grid
!$OMP THREADPRIVATE(flux_le_grid)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: u_speed_grid 
!$OMP THREADPRIVATE(u_speed_grid)

 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: t_a_next_grid
!$OMP THREADPRIVATE(t_a_next_grid)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: q_a_next_grid 
!$OMP THREADPRIVATE(q_a_next_grid)

 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: temp_atmos_pres_grid       ! atmospheric temperature down the column (K) PRESENT STEP
!$OMP THREADPRIVATE(temp_atmos_pres_grid)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: q_atmos_pres_grid          ! atmospheric specific humididy down the column (kg/kg) PRESENT STEP
!$OMP THREADPRIVATE(q_atmos_pres_grid)
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: temp_leaf_pres_grid        ! leaf temperature down the column (K) PRESENT STEP
!$OMP THREADPRIVATE(temp_leaf_pres_grid)

 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)        :: u_speed        ! canopy wind speed profile
!$OMP THREADPRIVATE(u_speed)



 ! these following two quantities are never calculated explicitly, they always have values passed from
 ! stomate...they need to be saved, though, since stomate is only called once a day
 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:,:) :: circ_class_biomass !! Stem diameter 
                                                                             !! @tex $(m)$ @endtex
!$OMP THREADPRIVATE(circ_class_biomass)

 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:)  :: circ_class_n       !! Number of trees within each circumference
!$OMP THREADPRIVATE(circ_class_n)


 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:,:) :: biomass                !! PFT total biomass 
!$OMP THREADPRIVATE(biomass)                                                  !! @tex $(gC m^{-2})$ @endtex

 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)    :: ind                    !! Number of individuals at the stand level
!$OMP THREADPRIVATE(ind)                                                      !! @tex $(m^{-2})$ @endtex

REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:,:)   :: lai_per_level   !! The LAI per vertical level
                                                                      !! @tex $(m^2 / m^2)$ @endtex
!$OMP THREADPRIVATE(lai_per_level)
REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: frac_snow_veg   !! The fraction of vegetation covered
                                                                      !! by snow.
                                                                      !! @tex $-$ @endtex
!$OMP THREADPRIVATE(frac_snow_veg)
REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)     :: frac_snow_nobio !! The fraction of non-bio land covered
                                                                      !! by snow.
                                                                      !! @tex $-$ @endtex
!$OMP THREADPRIVATE(frac_snow_nobio)
REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)       :: frac_snow_pix   !! The fraction of the whole pixel covered
                                                                      !! by snow.  This is computed from the above
                                                                      !! two.
                                                                      !! @tex $-$ @endtex
!$OMP THREADPRIVATE(frac_snow_pix)
  LOGICAL, SAVE                                    :: l_first_sechiba = .TRUE. !! Flag controlling the intialisation (true/false)

!$OMP THREADPRIVATE(l_first_sechiba)
  LOGICAL, SAVE                                    :: river_routing            !! Flag that decides if we route surface runoff 
                                                                               !! and deep drainage to the ocean through the 
                                                                               !! rivers
!$OMP THREADPRIVATE(river_routing)
  LOGICAL, SAVE                                    :: hydrol_cwrr              !! Flag that decides if we use the 11-layer 
                                                                               !! hydrology module. If not, Choisnel module is 
                                                                               !! taken.
!$OMP THREADPRIVATE(hydrol_cwrr)
  LOGICAL, SAVE                                    :: myfalse=.FALSE.          !! Local flag (true/false)
!$OMP THREADPRIVATE(myfalse)
  LOGICAL, SAVE                                    :: mytrue=.TRUE.            !! Local flag (true/false)
!$OMP THREADPRIVATE(mytrue)


! Variables related to snow processes calculations  

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)   :: snowrho      !! snow density for each layer
!$OMP THREADPRIVATE(snowrho)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)   :: snowheat     !! snow heat content for each layer (J/m2)
!$OMP THREADPRIVATE(snowheat)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)   :: snowliq      !! liquid water content (m)
!$OMP THREADPRIVATE(snowliq)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)   :: snowgrain    !! snow grain size (m)
!$OMP THREADPRIVATE(snowgrain)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)   :: snowtemp     !! snow temperature profile (K)
!$OMP THREADPRIVATE(snowtemp)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)   :: soiltemp     !! soil temperature profile in thermal layers (K)
!$OMP THREADPRIVATE(soiltemp)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)   :: snowdz       !! snow layer thickness (m)
!$OMP THREADPRIVATE(snowdz)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:,:)   :: radsink      !! radiation sink/source (J/m2)
!$OMP THREADPRIVATE(radsink)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)     :: grndflux     !! net energy into soil (W/m2)
!$OMP THREADPRIVATE(grndflux)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)     :: radxs        !! Residual radiation at the bottom of the snowpack (W/m2)
!$OMP THREADPRIVATE(radxs)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)     :: sfcfrz       !! snow surface layer frozen fraction
!$OMP THREADPRIVATE(sfcfrz)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)     :: gthick       !! soil surface layer thickness
!$OMP THREADPRIVATE(gthick)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)     :: gtemp        !! soil surface temperature
!$OMP THREADPRIVATE(gtemp)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)     :: soilflxresid !! Net flux to the snowpack
!$OMP THREADPRIVATE(soilflxresid)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION(:)     :: gpkappa      !! soil surface conductivity
!$OMP THREADPRIVATE(gpkappa)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: pgflux       !! net energy into snow pack
!$OMP THREADPRIVATE(pgflux)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: snowdz_o     !! snow depth at previous time step
!$OMP THREADPRIVATE(snowdz_o)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: albedo_undersnow !! albedo under the snowpack
!$OMP THREADPRIVATE(albedo_undersnow)
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: pkappa_snow      !! snow thermal conductivity
!$OMP THREADPRIVATE(pkappa_snow)
  LOGICAL, ALLOCATABLE, SAVE, DIMENSION(:)         :: osfcmelt         !! Indicate snow melting in each gridcell
!$OMP THREADPRIVATE(osfcmelt)


  LOGICAL, SAVE                                    :: james_netcdf_flag = .TRUE.      !! Flag that controls the activation of alternate enerbil scheme (true/false)
!$OMP THREADPRIVATE(james_netcdf_flag)
  INTEGER, SAVE                                    :: james_count = 0          !! count how many times to run jdriver
!$OMP THREADPRIVATE(james_count)
  LOGICAL, SAVE                                    :: hydrol_flag = .TRUE.    !! Flag that recalculates the energy budget for the new supply term (true/false)    
!$OMP THREADPRIVATE(hydrol_flag)
  
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)    :: sum_veget_diff          !! The is the difference between the total
                                                                              !! vegetation fraction in the new and old
                                                                              !! land cover maps.  Useful for land cover
                                                                              !! changes. [-]
!$OMP THREADPRIVATE(sum_veget_diff)


  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:,:) :: h_array_out         !! heights of tree levels from stomate           
!$OMP THREADPRIVATE(h_array_out)

 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:,:,:)  :: z_array_out         !! heights of tree levels from stomate           
!$OMP THREADPRIVATE(z_array_out)

 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:)        :: t2m_month_out    !! t2m longterm temperature from stomate           
!$OMP THREADPRIVATE(t2m_month_out)

 REAL(r_std),ALLOCATABLE, SAVE, DIMENSION (:, :, :)   :: profile_vbeta3
!$OMP THREADPRIVATE(profile_vbeta3)

 REAL(r_std),ALLOCATABLE, SAVE, DIMENSION (:, :, :)   :: profile_rveget
!$OMP THREADPRIVATE(profile_rveget)

 REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)      :: max_height_store     !! Same as z_array, but one less dimension.
                                                                              !! @tex $(m)$ @endte
!$OMP THREADPRIVATE(max_height_store)

  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: delta_c13_assim         !! C13 concentration in delta notation                                                                      !!                                                                              !! @tex $ permille $ @endtex (per thousand)   
!$OMP THREADPRIVATE(delta_c13_assim)
  
  REAL(r_std), ALLOCATABLE, SAVE, DIMENSION (:,:)  :: leaf_ci_out             !! Ci Leaf internal CO2 concentration ()      
!$OMP THREADPRIVATE(Ci)

CONTAINS

!! ==============================================================================================================================\n
!! SUBROUTINE 	: sechiba_main
!!
!>\BRIEF        Main routine for the sechiba module performing three functions:
!! variable initialization (during the first call only), calculating temporal
!! evolution of all variables (every call including the first call) and preparation
!! of output and restart files (during the last call only)
!!
!!\n DESCRIPTION : Main routine for the sechiba module. This module is called 
!! two times:
!! - a first time to set initial values
!! - a second time to compute the entire algorithm.\n 
!! Every time the module is called, three major if/then loops are checked:
!! - initialization (first call only),
!! - time step evolution of all variables (every call including the first),
!! - preparation of output and storage for the restart arrays (last call only).\n
!! One time step evolution consists of:
!! - call sechiba_var_init to do some initialization,
!! - call slowproc_main to do some daily initialization,
!! - call diffuco_main for diffusion coefficient calculation,
!! - call enerbil_main for energy budget calculation,
!! - call hydrolc_main (or hydrol_main) for hydrologic processes calculation,
!! - call enerbil_fusion : last part with fusion,
!! - call condveg_main for surface conditions such as roughness, albedo, and emmisivity,
!! - call thermosoil_main for soil thermodynamic calculation,
!! - call sechiba_end to swap previous to new fields.
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): Hydrological variables (:: coastalflow and :: riverflow),
!! components of the energy budget (:: tsol_rad, :: vevapp, :: fluxsens, 
!! :: temp_sol_new and :: fluxlat), surface characteristics (:: z0_out, :: emis_out, 
!! :: tq_cdrag and :: albedo_out) and land use related CO2 fluxes (:: netco2flux and 
!! :: fco2_lu)            
!!
!! REFERENCE(S)	: 
!!
!! FLOWCHART    : 
!! \latexonly 
!! \includegraphics[scale = 0.5]{sechibamainflow.png}
!! \endlatexonly
!! \n
!_ ================================================================================================================================

  SUBROUTINE sechiba_main (kjit, kjpij, kjpindex, index, dtradia, date0, &
       & ldrestart_read, ldrestart_write, control_in, &
       & lalo, contfrac, neighbours, resolution,&
       ! First level conditions
       & zlev, u, v, qair, q2m, t2m, temp_air, epot_air, ccanopy, &
       ! Variables for the implicit coupling
       & tq_cdrag, petAcoef, peqAcoef, petBcoef, peqBcoef, &
       ! Rain, snow, radiation and surface pressure
       & precip_rain, precip_snow, lwdown, swnet, swdown, sinang, pb, &
       ! Output : Fluxes
       & vevapp, fluxsens, fluxlat, coastalflow, riverflow, netco2flux, fco2_lu, &
       ! Surface temperatures and surface properties
       & tsol_rad, temp_sol_new, qsurf_out, albedo_out, emis_out, nlevelsloc_out, z0_out, &
       ! File ids
       & rest_id, hist_id, hist2_id, rest_id_stom, hist_id_stom, hist_id_stom_IPCC)

!! 0.1 Input variables
    
    INTEGER(i_std), INTENT(in)                               :: kjit              !! Time step number (unitless)
    INTEGER(i_std), INTENT(in)                               :: kjpij             !! Total size of the un-compressed grid 
                                                                                  !! (unitless)
    INTEGER(i_std), INTENT(in)                               :: kjpindex          !! Domain size - terrestrial pixels only 
                                                                                  !! (unitless)
    INTEGER(i_std),INTENT (in)                               :: rest_id           !! _Restart_ file identifier (unitless)
    INTEGER(i_std),INTENT (in)                               :: hist_id           !! _History_ file identifier (unitless)
    INTEGER(i_std),INTENT (in)                               :: hist2_id          !! _History_ file 2 identifier (unitless)
    INTEGER(i_std),INTENT (in)                               :: rest_id_stom      !! STOMATE's _Restart_ file identifier 
                                                                                  !! (unitless)
    INTEGER(i_std),INTENT (in)                               :: hist_id_stom      !! STOMATE's _History_ file identifier 
                                                                                  !! (unitless)
    INTEGER(i_std),INTENT(in)                                :: hist_id_stom_IPCC !! STOMATE's IPCC _history_ file file 
                                                                                  !! identifier (unitless)
    REAL(r_std), INTENT (in)                                 :: dtradia           !! Time step (s)
    REAL(r_std), INTENT (in)                                 :: date0             !! Initial date (??unit??)
    LOGICAL, INTENT(in)                                      :: ldrestart_read    !! Logical for _restart_ file to read 
                                                                                  !! (true/false)
    LOGICAL, INTENT(in)                                      :: ldrestart_write   !! Logical for _restart_ file to write 
                                                                                  !! (true/false)
    TYPE(control_type), INTENT(in)                           :: control_in        !! Flags that (de)activate parts of the model
    REAL(r_std),DIMENSION (kjpindex,2), INTENT (in)          :: lalo              !! Geographic coordinates (latitude,longitude)
                                                                                  !! for grid cells (degrees)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: contfrac          !! Fraction of continent in the grid 
                                                                                  !! (unitless, 0-1)
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)         :: index             !! Indices of the pixels on the map. 
                                                                                  !! Sechiba uses a reduced grid excluding oceans
                                                                                  !! ::index contains the indices of the 
                                                                                  !! terrestrial pixels only! (unitless)
    INTEGER(i_std), DIMENSION (kjpindex,8), INTENT(in)       :: neighbours        !! Neighboring grid points if land!(unitless)
    REAL(r_std), DIMENSION (kjpindex,2), INTENT(in)          :: resolution        !! Size in x and y of the grid (m)
    
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: u                 !! Lowest level wind speed in direction u 
                                                                                  !! @tex $(m.s^{-1})$ @endtex 
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: v                 !! Lowest level wind speed in direction v 
                                                                                  !! @tex $(m.s^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: zlev              !! Height of first layer (m)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: qair              !! Lowest level specific humidity 
                                                                                  !! @tex $(kg kg^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: q2m               !! 2m specific humidity 
                                                                                  !! @tex $(kg kg^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: t2m               !! 2m air temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: precip_rain       !! Rain precipitation 
                                                                                  !! @tex $(kg m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: precip_snow       !! Snow precipitation 
                                                                                  !! @tex $(kg m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: lwdown            !! Down-welling long-wave flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: sinang            !! Sine of the solar angle (unitless)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: swnet             !! Net surface short-wave flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: swdown            !! Down-welling surface short-wave flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: temp_air          !! Air temperature (K)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: epot_air          !! Air potential energy (??J)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: ccanopy           !! CO2 concentration in the canopy (ppm)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: petAcoef          !! Coefficients A for T from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: peqAcoef          !! Coefficients A for q from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: petBcoef          !! Coefficients B for T from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: peqBcoef          !! Coefficients B for q from the Planetary 
                                                                                  !! Boundary Layer
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)            :: pb                !! Lowest level pressure (hPa)
    
    REAL(r_std),DIMENSION (kjpindex)                         :: beta_diffuco



!! 0.2 Output variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: coastalflow       !! Outflow on coastal points by small basins.
                                                                                  !! This is the water which flows in a disperse 
                                                                                  !! way into the ocean
                                                                                  !! @tex $(kg dt_routing^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: riverflow         !! Outflow of the major rivers.
                                                                                  !! The flux will be located on the continental 
                                                                                  !! grid but this should be a coastal point  
                                                                                  !! @tex $(kg dt_routing^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: tsol_rad          !! Radiative surface temperature 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: vevapp            !! Total of evaporation 
                                                                                  !! @tex $(kg m^{-2} days^{-1})$ @endtex
    
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: qsurf_out         !! Surface specific humidity 
                                                                                  !! @tex $(kg kg^{-1})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: z0_out            !! Surface roughness (output diagnostic, m)
    REAL(r_std),DIMENSION (kjpindex,2), INTENT (out)         :: albedo_out        !! VIS and NIR albedo (output diagnostic, 
                                                                                  !! unitless)
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fluxsens          !! Sensible heat flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fluxlat           !! Latent heat flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: emis_out          !! Emissivity (unitless)

    INTEGER(i_std),DIMENSION (kjpindex), INTENT (out)        :: nlevelsloc_out    !! (JR) the number of vegetation levels in the canopy

    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: netco2flux        !! Sum CO2 flux over PFTs 
                                                                                  !! ??(gC m^{-2} s^{-1})??
    REAL(r_std),DIMENSION (kjpindex), INTENT (out)           :: fco2_lu           !! Land Cover Change CO2 flux 
    
    REAL(r_std),DIMENSION (kjpindex)                         :: netrad_out        !! Net radiation flux add by YC 20140313
    REAL(r_std),DIMENSION (kjpindex)                         :: soilflx_out       !! Soil heat flux add by YC 20140313
    
    LOGICAL, DIMENSION (kjpindex)                            :: hydrol_flag2      !! flag that 'trips' the alternative energy budget for each grid square
    LOGICAL, DIMENSION (kjpindex, nvm)                       :: hydrol_flag3      !! flag that 'trips' the alternative energy budget for each grid square

    LOGICAL, DIMENSION (kjpindex, nlevels_tot)               :: hydrol_flag2_column      !! flag that 'trips' the alternative energy budget for each grid square
    LOGICAL, DIMENSION (kjpindex, nvm, nlevels_tot)          :: hydrol_flag3_column      !! flag that 'trips' the alternative energy budget for each grid square

   ! REAL(r_std), DIMENSION(kjpindex,nvm), INTENT (out)      :: transpir_mod      !! transpir converted from mm/day to mm dt^(-1)

!! 0.3 Modified



    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: tq_cdrag                 !! Surface drag coefficient 
                                                                                         !! @tex $(m.s^{-1})$ @endtex 
    REAL(r_std),DIMENSION (kjpindex), INTENT (inout)         :: temp_sol_new             !! New ground temperature (K)

!! 0.4 local variables

    REAL(r_std), DIMENSION (kjpindex,nvm,nlai)               :: leaf_ci                  !! intercellular CO2 concentration (ppm)
    INTEGER(i_std)                                           :: ji, jv, icir,ilevel      !! Index (unitless)

    REAL(r_std), ALLOCATABLE, DIMENSION (:)                  :: runoff1           !! ??Temporary surface runoff calculated by 
                                                                                  !! hydrol @tex $(kg m^{-2})$ @endtex
    REAL(r_std), ALLOCATABLE, DIMENSION (:)                  :: drainage1         !! ??Temporary deep drainage calculatedd by 
                                                                                  !! hydrol @tex $(kg m^{-2})$ @endtex
    REAL(r_std), ALLOCATABLE, DIMENSION (:)                  :: soilcap1          !! ??Temporary soil heat capacity 
                                                                                  !! @tex $(J K^{-1})$ @endtex
    REAL(r_std), ALLOCATABLE, DIMENSION (:)                  :: soilflx1          !! ??Temporary soil heat flux 
                                                                                  !! @tex $(W m^{-2})$ @endtex
    REAL(r_std), ALLOCATABLE, DIMENSION (:,:)                :: shumdiag1         !! ??Temporary relative soil moisture 
                                                                                  !! (unitless, 0-1)
    REAL(r_std), DIMENSION(kjpindex)                         :: histvar           !! Computations for history files (unitless)
    CHARACTER(LEN=80)                                        :: var_name          !! To store variables names for I/O (unitless)
    REAL(r_std), DIMENSION(kjpindex)                         :: sum_treefrac      !! Total fraction occupied by trees (0-1, uniless) 
    REAL(r_std), DIMENSION(kjpindex)                         :: sum_grassfrac     !! Total fraction occupied by grasses (0-1, unitless)
    REAL(r_std), DIMENSION(kjpindex)                         :: sum_cropfrac      !! Total fraction occcupied by crops (0-1, unitess)
!    REAL(r_std), ALLOCATABLE, DIMENSION (:,:,:)             :: swc               !! soil water content m3 m-3 (copy of mc in hydrol_main)
    !REAL(r_std), DIMENSION (kjpindex,nslm,nstm)             :: swc               !! soil water content m3 m-3 (copy of mc in hydrol_main)
    INTEGER(i_std), DIMENSION(kjpindex,nvm,nstm)             :: veg_soiltext      !! Percentage of each veg. type on each soil type
                                                                                  !! of each grid point (copy of corr_veg_soil)
    LOGICAL                                                  :: hydrol_flag       !! flag that 'trips' the alternative energy budget for each grid square
    LOGICAL                                                  :: hydrol_flag_column       !! flag that 'trips' the alternative energy budget for each grid square


    REAL(r_std)                                              :: isotrop_angle     

    REAL(r_std),DIMENSION (kjpindex,nvm)                     :: vbeta23           !! Beta for fraction of wetted foliage that will
                                                                                  !! transpire once intercepted water has evaporated (-)

    !+++++ TEMP++++++
    ! for printing
    REAL(r_std), DIMENSION(kjpindex,nvm)                :: rdi
    REAL(r_std), DIMENSION(ncirc)                       :: diameters_temp
    INTEGER :: ivm,ipts
    REAL(r_std), DIMENSION(kjpindex)                    :: laieff_pix                 !! LAIeff at the pixel level for optimization
    REAL(r_std), DIMENSION(nlevels_tot,kjpindex,nvm)    :: laieff_isotrop             !! Effective LAI
    REAL(r_std), DIMENSION(kjpindex,nvm)                :: laieff_pft                 !! Effective LAI 
 

    ! %%% begin variables for the enerbil and hydrol interface %%%%

    INTEGER                                             :: i, inno
    INTEGER, SAVE                                       :: james_step_count         !$OMP THREADPRIVATE(james_step_count)
!$OMP THREADPRIVATE(james_step_count)

    REAL(r_std)                                         :: flux_ground_h            ! sensible heat flux at surface (W^{m^2})
    REAL(r_std)                                         :: flux_ground_le           ! latent heat flux at surface (W^{m^2})
    REAL(r_std)                                         :: temp_surface_next_out    ! check for surface temperature at next timestep (K)

    REAL(r_std), DIMENSION(kjpindex)                    :: vbeta2sum             ! sum of vbeta2 coefficients across all PFTs (-)
    REAL(r_std), DIMENSION(kjpindex)                    :: vbeta3sum             ! sum of vbete3 coefficients across all PFTs (-)

    REAL(r_std), SAVE                                   :: temp_surf_pres        ! temperature at surface for the present timestep (K)
!$OMP THREADPRIVATE(temp_surf_pres)
    REAL(r_std), SAVE                                   :: temp_surf_next        ! temperature at surface for the next timestep (K)
!$OMP THREADPRIVATE(temp_surf_next)

    REAL(r_std), DIMENSION (kjpindex,nvm,nlevels_tot)       :: gs_distribution
    REAL(r_std), DIMENSION (kjpindex,nvm,nlevels_tot)       :: gs_diffuco_output
    REAL(r_std), DIMENSION (kjpindex,nvm,nlevels_tot)       :: gstot_component
    REAL(r_std), DIMENSION (kjpindex,nvm,nlevels_tot)       :: gstot_frac


    ! variables introduced as a consequence of running enerbil twice (if needed, as a result of restriction of transpiration by
    !    the supply term). If running for the second time, we should start with the same input variables as the first run
    
    REAL(r_std), DIMENSION (kjpindex)        :: evapot_save           !! Soil Potential Evaporation (mm day^{-1})
    REAL(r_std), DIMENSION (kjpindex)        :: evapot_corr_save      !! Soil Potential Evaporation Correction (mm day^{-1})
    REAL(r_std), DIMENSION (kjpindex)        :: temp_sol_save         !! Soil temperature (K)
    REAL(r_std), DIMENSION (kjpindex)        :: qsurf_save            !! Surface specific humidity (kg kg^{-1})     
    REAL(r_std), DIMENSION (kjpindex)        :: fluxsens_save         !! Sensible heat flux (W m^{-2})
    REAL(r_std), DIMENSION (kjpindex)        :: fluxlat_save          !! Latent heat flux (W m^{-2})
    REAL(r_std), DIMENSION (kjpindex)        :: tsol_rad_save         !! Tsol_rad (W m^{-2})
    REAL(r_std), DIMENSION (kjpindex)        :: vevapp_save           !! Total of evaporation (mm day^{-1})
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: gpp_save              !! Assimilation ((gC m^{-2}), total area)     
    REAL(r_std), DIMENSION (kjpindex)        :: temp_sol_new_save     !! New soil temperature (K)
    REAL(r_std), DIMENSION (kjpindex)        :: vbeta2sum_save        !! sum of vbeta2 coefficients across all PFTs (-)
    REAL(r_std), DIMENSION (kjpindex)        :: vbeta3sum_save        !! sum of vbeta3 coefficients across all PFTs (-)
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: unstressed            !! Adjusted ecosystem functioning. Takes the unit of the variable
                                                                      !! used as a proxy for waterstress
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: stressed              !! Initial ecosystem functioning after the first calculation and 
                                                                      !! before any recalculations. Takes the unit of the variable used
                                                                      !! as a proxy for unstressed.
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: vbeta3_save           !! Beta for Transpiration (unitless)
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: vbeta3pot_save        !! Beta for Potential Transpiration
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: rveget_save           !! stomatal resistance of vegetation 
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: rstruct_save          !! structural resistance @tex ($s m^{-1}$) @endtex
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: cimean_save           !! mean intercellular CO2 concentration
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: gsmean_save           !! mean stomatal conductance to CO2 (umol m-2 s-1)
   ! REAL(r_std), DIMENSION (kjpindex,nvm)   :: gpp_save
    REAL(r_std), DIMENSION (kjpindex)        :: qsol_sat_new_out      !! New saturated surface air moisture (kg kg^{-1})
    REAL(r_std),DIMENSION (kjpindex)         :: qair_new
    REAL(r_std),DIMENSION (kjpindex)         :: delta_temp_save
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: vbeta3_oldvalue
    REAL(r_std), DIMENSION (kjpindex)        :: vbeta3_save_test
    
    REAL(r_std), DIMENSION (kjpindex,nvm)    :: transpir_init         !! Temperate storage variable for the transpiration before limited by transpiration supply  
  !  REAL(r_std),DIMENSION(kjpindex,nvm,nlevels_tot)    :: z_array3      !! Same as z_array, but one less dimension.
  !                                                                     !! @tex $(m)$ @endte

    REAL(r_std), DIMENSION(nlevels_tot,kjpindex,nvm)        :: laieff_collim          !! (FROM STOMATE) Leaf Area Index Effective for direct light

  !  REAL(r_std),DIMENSION(kjpindex,nvm,ncirc,nlevels_tot)   :: h_array_out             !! (FROM STOMATE) An output of h_array, to use in sechiba               
  !  REAL(r_std),DIMENSION(kjpindex,nvm,ncirc,nlevels_tot)   :: z_array_out             !! (FROM STOMATE) An output of h_array, to use in sechiba                                                                           

    REAL(r_std),DIMENSION (kjpindex,nvm,nlevels_tot)        :: profile_vbeta3
    REAL(r_std),DIMENSION (kjpindex,nvm,nlevels_tot)        :: profile_rveget

    REAL(r_std), DIMENSION(nlevels_tot, kjpindex, nvm)      :: transpir_supply_column   !! (SECHIBA TOP LEVEL) Supply of water for transpiration 

  !  REAL(r_std), DIMENSION (kjpindex,nvm)                   :: max_height_store     

    REAL(r_std),DIMENSION(nlevels_tot)                      :: Collim_Abs_Tot           !! (SECHIBA TOP LEVEL) collimated total absorption for a given level
    REAL(r_std),DIMENSION(nlevels_tot)                      :: Collim_Alb_Tot           !! (SECHIBA TOP LEVEL) Collimated (direct) total albedo for a given level

    REAL(r_std),DIMENSION(nlevels_tot)                      :: laieff_isotrop_pft       !! (FROM STOMATE) collimated total absorption for a given level
    REAL(r_std),DIMENSION(nlevels_tot)                      :: laieff_collim_pft        !! (FROM STOMATE) Collimated (direct) total albedo for a given level

 ! %%% end variables for the enerbil and hydrol interface %%%%
  
 
!_ ================================================================================================================================
!    WRITE(numout,*) '090515 sechiba, start of module h_array_out(1,4,1,:): ', h_array_out(1,4,1,:)   
!    WRITE(numout,*) '090515 sechiba, start of module h_array_out(1,4,2,:): ', h_array_out(1,4,2,:)   
!    WRITE(numout,*) '090515 sechiba, start of module h_array_out(1,4,3,:): ', h_array_out(1,4,3,:)   
!    WRITE(numout,*) '090515 sechiba, start of module z_array_out(1,4,1,:): ', z_array_out(1,4,1,:)
!    WRITE(numout,*) '090515 sechiba, start of module z_array_out(1,4,1,:): ', z_array_out(1,4,2,:)
!    WRITE(numout,*) '090515 sechiba, start of module z_array_out(1,4,1,:): ', z_array_out(1,4,3,:)
!    WRITE(numout,*) '090515 max_height_store(1,4): ', max_height_store
!    WRITE(numout,*) '230615 t2m_longterm in sechiba:', t2m_month_out(:)
!    WRITE(numout,*) '20150327 lwdown in sechiba:', lwdown(:) 
    
    IF (long_print) WRITE(numout,*) ' sechiba kjpindex =',kjpindex

!! 1. Initialize variables on first call!

    IF (l_first_sechiba) THEN

       !+++++++ TEMP +++++++++
       ! It is good to leave this in here.  It is only written out once,
       ! it takes almost no time, and it's necessary for identifying a 
       ! problem pixel.
       DO ipts=1,kjpindex
          WRITE(numout,'(A,I6,10F20.10)') 'lalo sechiba: ',ipts,lalo(ipts,1:2)
       ENDDO
       !++++++++++++++++
      
       james_step_count = 0
       ALLOCATE(jnlevels_loc(kjpindex)) ! (JR)
       !! 1.1 Initialize most of sechiba's variables
       CALL sechiba_init (kjit, ldrestart_read, kjpij, kjpindex, index, rest_id, control_in, lalo)
     
       ALLOCATE(runoff1 (kjpindex),drainage1 (kjpindex), soilcap1 (kjpindex),soilflx1 (kjpindex))
       ALLOCATE(shumdiag1(kjpindex,nbdl))
        
       ! These are data that are passed from stomate to sechiba for
       ! the effective LAI calculation...however it seems that they
       ! are not always initialized before the effective lai routine
       ! in condveg is called
       circ_class_biomass=val_exp
       circ_class_n=val_exp

       !! 1.2 Initialize some variables of energy budget from restart file     
       IF (ldrestart_read) THEN
          
          IF (long_print) WRITE (numout,*) ' we have to read a restart file for SECHIBA variables'
          var_name='soilcap' ;
          CALL ioconf_setatt_p('UNITS', '-')
          CALL ioconf_setatt_p('LONG_NAME','Soil calorific capacity')
          soilcap1=val_exp
          IF ( ok_var(var_name) ) THEN
             CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., soilcap1, "gather", nbp_glo, index_g)
             IF (MINVAL(soilcap1) < MAXVAL(soilcap1) .OR. MAXVAL(soilcap1) < val_exp) THEN
                soilcap(:) = soilcap1(:)
             ENDIF
          ENDIF
          
          var_name='soilflx' ;
          CALL ioconf_setatt_p('UNITS', '-')
          CALL ioconf_setatt_p('LONG_NAME','Soil flux')
          soilflx1=val_exp
          IF ( ok_var(var_name) ) THEN
             CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., soilflx1, "gather", nbp_glo, index_g)
             IF (MINVAL(soilflx1) < MAXVAL(soilflx1)  .OR. MAXVAL(soilflx1) < val_exp) THEN
                soilflx(:) = soilflx1(:)
             ENDIF
          ENDIF
          
          var_name='shumdiag' ;
          CALL ioconf_setatt_p('UNITS', '-')
          CALL ioconf_setatt_p('LONG_NAME','Relative soil moisture')
          shumdiag1=val_exp
          IF ( ok_var(var_name) ) THEN
             CALL restget_p (rest_id, var_name, nbp_glo, nbdl, 1, kjit, .TRUE., shumdiag1, "gather", nbp_glo, index_g)
             IF (MINVAL(shumdiag1) < MAXVAL(shumdiag1) .OR. MAXVAL(shumdiag1) < val_exp) THEN
                shumdiag(:,:) = shumdiag1(:,:)
             ENDIF
          ENDIF
       ENDIF ! ldrestart_read

       !! 1.3 Initialize stomate's variables
       ! This variable needs to be defined before going into slowproc.
       Isotrop_Tran_Tot_p(:,:,:)=zero
       CALL slowproc_main (kjit, kjpij, kjpindex, dtradia, date0, &
           ldrestart_read, ldrestart_write, control_in%ok_co2, control_in%ok_stomate, index, &
           indexveg, lalo, neighbours, resolution, contfrac, &
           soiltile, reinf_slope, t2mdiag, t2mdiag, temp_sol, &
           stempdiag, vegstress, shumdiag, litterhumdiag, precip_rain, &
           precip_snow, gpp, deadleaf_cover, assim_param, lai, &
           frac_age, height, veget, frac_nobio, njsc, &
           veget_max, totfrac_nobio, qsintmax, rest_id, hist_id, &
           hist2_id, rest_id_stom, hist_id_stom, hist_id_stom_IPCC, co2_flux, &
           fco2_lu, sinang, circ_class_biomass, circ_class_n, &
           transpir, stressed, unstressed, transpir_supply, &
           biomass, ind, lai_per_level, laieff_fit,  &
           vir_transpir_supply, temp_growth, Isotrop_Tran_Tot_p,&
           sum_veget_diff, snow, jnlevels_loc, h_array_out, z_array_out, &
           max_height_store, t2m_month_out)

       ALLOCATE (u_speed(jnlvls))

       netco2flux(:) = zero

       !+++CHECK+++
       ! If co2flux contains soil respiration the loop should include the bare soil
       ! because after a land cover change PFT1 can contain litter and soil carbon
       DO jv = 2,nvm
          netco2flux(:) = netco2flux(:) + co2_flux(:,jv)*veget_max(:,jv)
       ENDDO
       !+++++++++++
      
       !! 1.4 Initialize diffusion coefficients
       CALL diffuco_main (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, &
            & index, indexveg, indexlai, u, v, &
            & zlev, z0, roughheight, temp_sol, temp_air, &
            & temp_growth, rau, tq_cdrag, qsurf, qair, &
            & q2m, t2m, pb, rsol, evap_bare_lim, &
            & evapot, evapot_corr, snow, flood_frac, flood_res, &
            & frac_nobio, snow_nobio, totfrac_nobio, swnet, swdown, &
            & sinang, ccanopy, humrel, vegstress, veget, &
            & veget_max, biomass, qsintveg, qsintmax, assim_param, &
            & vbeta, valpha, vbeta1, vbeta2, vbeta3, &
            & vbeta3pot, vbeta4, vbeta5, gsmean, rveget, &
            & rstruct, cimean, gpp, lalo, neighbours, &
            & resolution, ptnlev1, precip_rain, frac_age, rest_id, &
            & hist_id, hist2_id, vbeta2sum, vbeta3sum, Isotrop_Abs_Tot_p, &
            & Isotrop_Tran_Tot_p, lai_per_level, lai, vbeta23, leaf_ci, &
            & gs_distribution, gs_diffuco_output, gstot_component, gstot_frac, warnings, &
            & u_speed, profile_vbeta3, profile_rveget, delta_c13_assim, leaf_ci_out)
       write(numout, *) "YC: Initialize remaining variables of energy budget"
       !! 1.5 Initialize remaining variables of energy budget
       !!       First calling "enerbil_main" for initializing variables
       CALL enerbil_main (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, &
            & index, indexveg, zlev, lwdown, swnet, swdown, &
            & epot_air, temp_air, u, v, petAcoef, &
            & petBcoef, qair, peqAcoef, peqBcoef, pb, &
            & rau, vbeta, valpha, vbeta1, vbeta2, &
            & vbeta3, vbeta3pot, vbeta4, vbeta5, emis, &
            & soilflx, soilcap, tq_cdrag, humrel, fluxsens, &
            & fluxlat, vevapp, transpir, transpot, vevapnu, &
            & vevapwet, vevapsno, vevapflo, t2mdiag, temp_sol, &
            & tsol_rad, temp_sol_new, qsurf, evapot, evapot_corr, &
            & rest_id, hist_id, hist2_id, beta_diffuco, &
            & precip_rain, snow, sfcfrz, radsink, &
            & veget_max, osfcmelt, pgflux, gthick, gtemp, &
            & gpkappa, grndflux, albedo_undersnow, snowtemp, snowdz, &
            & radxs, pkappa_snow, precip_snow, snowliq, snowheat, & 
            & snowrho, snowgrain, snowdz_o, soilflxresid)

       ! initialise some multilayer energy budget variables 
        
       temp_leaf_pres_grid(:,:) = 300.0d0  ! this is the leaf temperature for the first time step 
                                           !     (for the multi-level energy budget)

       DO i = 1, kjpindex
          temp_atmos_pres_grid(i,:) = temp_air(i)
          q_atmos_pres_grid(i,:) = qair(i)
       END DO ! i = 1, kjpindex


       ! for sechiba_hydrol_arch
       swc(:,:,:) = 0.0d0
      
       IF (control%ok_new_enerbil_nextstep ) THEN 
       
		   ! for enerbil nextstep initialisation
	   
		   ! we have a problem in that z_array_out has a value of zero on first call of slowproc_main (above).
		   ! so we define z_array_out with temporary values
		   u_speed(:) = 0.0d0
		   z_array_out(:,:,:,:) = 0.0d0
		   h_array_out = 00.0d0                    
		   z_array_out(:,:,:,1) = 0.0d0        
		   z_array_out(:,:,:,2) = 2.0d0        
		   z_array_out(:,:,:,3) = 4.0d0        
		   z_array_out(:,:,:,4) = 6.0d0        
		   z_array_out(:,:,:,5) = 8.0d0        
		   z_array_out(:,:,:,6) = 10.0d0        
		   z_array_out(:,:,:,7) = 12.0d0        
		   z_array_out(:,:,:,8) = 14.0d0        
		   z_array_out(:,:,:,9) = 16.0d0        
		   z_array_out(:,:,:,10) = 18.0d0 

		   laieff_isotrop(:,:, :) = 0.0d0

		   collim_abs_tot(:) = 0.0d0
		   Collim_Alb_Tot(:) = 0.0d0

		   max_height_store = 20.0d0
        
       END IF ! (control%ok_new_enerbil_nextstep ) THEN 


       
       !! 1.6 Initialize some hydrological variables from restart file
       IF (ldrestart_read) THEN
          
          var_name='runoff' ;
          CALL ioconf_setatt_p('UNITS', 'mm/d')
          CALL ioconf_setatt_p('LONG_NAME','Complete runoff')
          runoff1=val_exp
          IF ( ok_var(var_name) ) THEN
             CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., runoff1, "gather", nbp_glo, index_g)
             IF (MINVAL(runoff1) < MAXVAL(runoff1) .OR. MAXVAL(runoff1) < val_exp) THEN
                runoff(:) = runoff1(:)
             ENDIF
          ENDIF

          var_name='drainage' ;
          CALL ioconf_setatt_p('UNITS', 'mm/d')
          CALL ioconf_setatt_p('LONG_NAME','Deep drainage')
          drainage1=val_exp
          IF ( ok_var(var_name) ) THEN
             CALL restget_p (rest_id, var_name, nbp_glo, 1, 1, kjit, .TRUE., drainage1, "gather", nbp_glo, index_g)
             IF (MINVAL(drainage1) < MAXVAL(drainage1) .OR. MAXVAL(drainage1) < val_exp) THEN
                drainage(:) = drainage1(:)
             ENDIF
          ENDIF

          IF ( ok_var("shumdiag") ) THEN
             IF (MINVAL(shumdiag1) < MAXVAL(shumdiag1) .OR. MAXVAL(shumdiag1) < val_exp) THEN
                shumdiag(:,:) = shumdiag1(:,:)
             ENDIF
          ENDIF
       ENDIF ! ldrestart_read
       
       !! 1.7 Initialize remaining hydrological variables
       IF ( .NOT. hydrol_cwrr ) THEN
          ! 1.7.1 Initialize remaining hydrological variables from Choisnel module (2 soil layers)
          CALL hydrolc_main (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, &
               & index, indexveg, control_in, temp_sol_new, floodout, &
               & runoff, drainage, frac_nobio, totfrac_nobio, vevapwet, &
               & veget, veget_max, qsintmax, qsintveg, vevapnu, &
               & vevapsno, vevapflo, snow, snow_age, snow_nobio, &
               & snow_nobio_age, tot_melt, transpir, precip_rain, precip_snow, &
               & returnflow, reinfiltration, irrigation, humrel, vegstress, &
               & rsol, drysoil_frac, evapot, evapot_corr, flood_frac, &
               & flood_res, shumdiag, litterhumdiag, soilcap, rest_id, &
               & hist_id, hist2_id, temp_air, pb, u, v, &
               & swnet, albedo_undersnow, pgflux, snowrho, snowtemp, &
               & soiltemp, snowgrain, snowdz, snowheat, snowliq, &
               & sfcfrz, osfcmelt, grndflux, radsink, radxs, &
               & gtemp, soilflxresid) 

          evap_bare_lim(:) = -un
          k_litt(:) = huit
          ! No specific calculation for shumdiag_perma. We assume it to shumdiag. 
          shumdiag_perma(:,:)=shumdiag(:,:)
       ELSE
          !! 1.7.2 Initialize remaining hydrological variables from CWRR module (11 soil layers)
          CALL hydrol_main (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, &
               & index, indexveg, indexsoil, indexlayer, indexnbdl, &
               & control_in, temp_sol_new, floodout, runoff, drainage, &
               & frac_nobio, totfrac_nobio, vevapwet, veget, veget_max, &
               & njsc, qsintmax, qsintveg, vevapnu, vevapsno, &
               & vevapflo, snow, snow_age, snow_nobio, snow_nobio_age, &
               & tot_melt, transpir, precip_rain, precip_snow, returnflow, &
               & reinfiltration, irrigation, humrel, vegstress, drysoil_frac, &
               & evapot, evapot_corr, evap_bare_lim, flood_frac, flood_res, &
               & shumdiag, shumdiag_perma, k_litt, litterhumdiag, soilcap, &
               & soiltile, reinf_slope, rest_id, hist_id, hist2_id, &
               & swc, veg_soiltext, stempdiag, temp_air, pb, &
               & u, v, swnet, albedo_undersnow, pgflux, &
               & snowrho, snowtemp, soiltemp, snowgrain, snowdz, &
               & snowheat, snowliq, sfcfrz, grndflux, radsink, &
               & radxs, gtemp, soilflxresid)
       ENDIF !hydrol_cwrr

       !! 1.8 Initialize some water balance variables from restart file
       IF (ldrestart_read) THEN
          
          IF ( ok_var("runoff") ) THEN
             IF (MINVAL(runoff1) < MAXVAL(runoff1) .OR. MAXVAL(runoff1) < val_exp) THEN
                runoff(:) = runoff1(:)
             ENDIF
          ENDIF

          IF ( ok_var("drainage") ) THEN
             IF (MINVAL(drainage1) < MAXVAL(drainage1) .OR. MAXVAL(drainage1) < val_exp) THEN
                drainage(:) = drainage1(:)
             ENDIF
          ENDIF
          
          IF ( ok_var("shumdiag") ) THEN
             IF (MINVAL(shumdiag1) < MAXVAL(shumdiag1) .OR. MAXVAL(shumdiag1) < val_exp) THEN
                shumdiag(:,:) = shumdiag1(:,:)
             ENDIF
          ENDIF
       ENDIF ! ldrestart_read

       !! 1.9 Initialize surface parameters (emissivity, albedo and roughness)
       CALL condveg_main (kjit, kjpindex, dtradia, ldrestart_read, &
            ldrestart_write, index, lalo, neighbours, resolution, contfrac, &
            veget, veget_max, frac_nobio, totfrac_nobio, zlev, &
            snow, snow_age, snow_nobio, snow_nobio_age, drysoil_frac, &
            height,  emis, albedo, z0, roughheight, &
            rest_id, hist_id, hist2_id, lai,  sinang, &
            circ_class_biomass, circ_class_n, lai_split, z0_veg, Isotrop_Abs_Tot_p, &
            Isotrop_Tran_Tot_p, laieff_fit, albedo_pft, frac_snow_nobio, frac_snow_veg, &
            albedo_undersnow, snowdz, snowrho, snowliq, precip_snow, &
            osfcmelt, temp_air, u, v, &
            Collim_Abs_Tot, Collim_Alb_Tot, laieff_collim, laieff_isotrop)

       !! 1.10 Initialization of soil thermodynamics
       CALL thermosoil_main (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, &
            & index, indexgrnd,indexnbdl, control_in,temp_sol_new, snow, soilcap, soilflx, & 
            & shumdiag_perma, stempdiag, ptnlev1, rest_id, hist_id, hist2_id, & 
            & soiltemp,pb,grndflux,snowrho,snowdz,snowtemp,gthick,gtemp,gpkappa,& 
            & pkappa_snow,snowdz_o)  

       !! 1.11 Initialize some soil thermodynamics from restart file
       IF (ldrestart_read) THEN
          IF ( ok_var("soilcap") ) THEN
             IF (MINVAL(soilcap1) < MAXVAL(soilcap1) .OR. MAXVAL(soilcap1) < val_exp) THEN
                soilcap(:) = soilcap1(:)
             ENDIF
          ENDIF
         
          IF ( ok_var("soilflx") ) THEN
             IF (MINVAL(soilflx1) < MAXVAL(soilflx1)  .OR. MAXVAL(soilflx1) < val_exp) THEN
                soilflx(:) = soilflx1(:)
             ENDIF
          ENDIF
       ENDIF ! ldrestart_read

       !! 1.12 Initialize river routing
       IF ( river_routing .AND. nbp_glo .GT. 1) THEN
          !! 1.12.1 Initialize river routing
          CALL routing_main (kjit, kjpindex, dtradia, control_in, ldrestart_read, ldrestart_write, index, &
               & lalo, neighbours, resolution, contfrac, totfrac_nobio, veget_max, floodout, runoff, &
               & drainage, transpot, precip_rain, humrel, k_litt, flood_frac, flood_res, &
               & stempdiag, reinf_slope, returnflow, reinfiltration, irrigation, riverflow, coastalflow, &
               & rest_id, hist_id, hist2_id)
       ELSE
          !! 1.12.2 No routing, set variables to zero
          riverflow(:) = zero
          coastalflow(:) = zero
          returnflow(:) = zero
          reinfiltration(:) = zero
          irrigation(:) = zero
          flood_frac(:) = zero
          flood_res(:) = zero
       ENDIF

       !! 1.13 Write internal variables to output fields
       z0_out(:) = z0(:)
       emis_out(:) = emis(:)
       albedo_out(:,:) = albedo(:,:) 
       qsurf_out(:) = qsurf(:)
       nlevelsloc_out(:) = jnlevels_loc(:)

       !! 1.14 Deallocate memory
       DEALLOCATE(runoff1,drainage1,soilcap1,soilflx1)
       DEALLOCATE(shumdiag1)
       

       !! 1.15 Some variables need to be initialized here, since
       !! slowproc is not called in this loop.  These variables
       !! are used in diffuco, which is called before slowproc.
       temp_growth(:)=zero

       RETURN ! Out of sechiba main (should remain the last line!)
 

    ENDIF ! l_first_sechiba




!! 2. Computes SECHIBA's variables

    !! 2.1 Initialize variables at each time step
    CALL sechiba_var_init (kjpindex, rau, pb, temp_air) 




    !! 2.2 Compute diffusion coefficients
    CALL diffuco_main (kjit, kjpindex, dtradia, ldrestart_read, myfalse, &
         & index, indexveg, indexlai, u, v, &
         & zlev, z0, roughheight, temp_sol, temp_air, &
         & temp_growth, rau, tq_cdrag, qsurf, qair, &
         & q2m, t2m, pb, rsol, evap_bare_lim, &
         & evapot, evapot_corr, snow, flood_frac, flood_res, &
         & frac_nobio, snow_nobio, totfrac_nobio, swnet, swdown, &
         & sinang, ccanopy, humrel, vegstress, veget, &
         & veget_max, biomass, qsintveg, qsintmax, assim_param, &
         & vbeta, valpha, vbeta1, vbeta2, vbeta3, &
         & vbeta3pot, vbeta4, vbeta5, gsmean, rveget, &
         & rstruct, cimean, gpp, lalo, neighbours, &
         & resolution, ptnlev1, precip_rain, frac_age, rest_id, &
         & hist_id, hist2_id, vbeta2sum, vbeta3sum, Isotrop_Abs_Tot_p, &
         & Isotrop_Tran_Tot_p, lai_per_level, lai, vbeta23, leaf_ci, &
         & gs_distribution, gs_diffuco_output, gstot_component, gstot_frac, warnings, &
         & u_speed, profile_vbeta3, profile_rveget, delta_c13_assim, leaf_ci_out)

    !! 2.3 Compute energy balance

    ! IF (control_in%ok_nenerbil) THEN
    IF (control%ok_nenerbil) THEN

       hydrol_flag = .FALSE.
       hydrol_flag2 = .FALSE.
       hydrol_flag3 = .FALSE.
       
       ! ---------------------------------------------------------------------------------------------------
       ! before first calling diffuco_trans_co2 again, store output values from the first call to diffuco
  
       vbeta3_save(:,:) = vbeta3(:,:)           !! Beta for Transpiration (unitless)
       vbeta3pot_save(:,:) = vbeta3pot(:,:)     !! Beta for Potential Transpiration
       rveget_save(:,:) = rveget(:,:)           !! stomatal resistance of vegetation 
       rstruct_save(:,:) = rstruct(:,:)         !! structural resistance @tex ($s m^{-1}$) @endtex
       cimean_save(:,:) = cimean(:,:)           ! mean intercellular CO2 concentration
       gsmean_save(:,:) = gsmean(:,:)           ! mean stomatal conductance to CO2 (umol m-2 s-1)
       gpp_save(:,:) = gpp(:,:)                 !! gross primary production
       ! ---------------------------------------------------------------------------------------------------

       ! WRITE(numout,*) '100114 before copy TO temp_sol_new_save, temp_sol_new(1) is: ', temp_sol_new(1)
       ! WRITE(numout,*) '100114 before copy TO temp_sol_new_save, temp_sol_new(2) is: ', temp_sol_new(2)
       
       ! ---------------------------------------------------------------------------------------------------
       ! before first call to enerbil, store initial values
  
        evapot_save(:) = evapot(:)                !! Soil Potential Evaporation (mm day^{-1})
        evapot_corr_save(:) = evapot_corr(:)      !! Soil Potential Evaporation Correction (mm day^{-1})
        qsurf_save(:) = qsurf(:)                  !! Surface specific humidity (kg kg^{-1})

        fluxsens_save(:) = fluxsens(:)            !! Sensible heat flux (W m^{-2})
        fluxlat_save(:) = fluxlat(:)              !! Latent heat flux (W m^{-2})

        temp_sol_save(:) = temp_sol(:)            !! Soil temperature (K)
        temp_sol_new_save(:) = temp_sol_new(:)    !! New soil temperature (K)
        tsol_rad_save(:) = tsol_rad(:)            !! Tsol_rad (W m^{-2})
        vevapp_save(:) = vevapp(:)                !! Total of evaporation (mm day^{-1})
        gpp_save(:,:) = gpp(:,:)                  !! Assimilation ((gC m^{-2}), total area)
        vbeta2sum_save(:) = vbeta2sum(:) 
        vbeta3sum_save(:) = vbeta3sum(:)
        ! ---------------------------------------------------------------------------------------------------

        IF (ld_jameswrite) THEN
            WRITE(numout,*) '260215, in sechiba, before jdriver, swnet is = ', swnet 
            WRITE(numout, *) '020315b in sechiba, SUM(laieff_isotrop(1:jnlvls_canopy, 1, 4) is: ', SUM(laieff_isotrop(1:jnlvls_canopy, 1, 4))
            WRITE(numout, *) '130315 before jdriver'                 
        END IF ! (ld_jameswrite)

        CALL jdriver (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, &
             & index, indexveg, zlev, lwdown, swnet, swdown, epot_air, temp_air, &
             & u, v, petAcoef, petBcoef, qair, peqAcoef, peqBcoef, pb, rau, &
             & vbeta, valpha, vbeta1, vbeta2, vbeta3, vbeta3pot, vbeta4, vbeta5, &
             & emis, soilflx, soilcap, tq_cdrag, humrel, &
!!$             & cimean, ccanopy, emis, soilflx, soilcap, tq_cdrag, humrel, &
             & fluxsens, fluxlat, vevapp, transpir, transpot, vevapnu, vevapwet, &
!!$             & fluxsens, fluxlat, vevapp, transpir, transpot, gpp, vevapnu, vevapwet, &
             & vevapsno, vevapflo, t2mdiag, temp_sol, tsol_rad, temp_sol_new, qsurf, &
             & evapot, evapot_corr, rest_id, hist_id, hist2_id, & 
             & flux_ground_h, flux_ground_le, flux_h_grid, flux_le_grid, t_a_next_grid, q_a_next_grid, & 
             & temp_surf_next, temp_atmos_pres_grid, q_atmos_pres_grid, &
             & james_netcdf_flag, james_step_count, temp_leaf_pres_grid, temp_surf_pres, &
             & transpir_supply, hydrol_flag, hydrol_flag2, &
             & hydrol_flag3, vbeta2sum, vbeta3sum, veget_max, qsol_sat_new_out, qair_new, delta_temp_save, netrad_out, veget, &
             & Collim_Abs_Tot, Collim_Alb_Tot, laieff_isotrop, &
             & h_array_out, z_array_out, transpir_supply_column, u_speed, profile_vbeta3, profile_rveget, max_height_store, &
             & u_speed_grid, flux_rn_grid, t2m_month_out )   

        delta_temp_save = temp_surf_next - temp_sol

        ! JR 260314 these are calculated in the first call to the jdriver, but may be required if it is called again
        fluxsens_save(:) = fluxsens(:)            !! Sensible heat flux (W m^{-2})
        fluxlat_save(:) = fluxlat(:)              !! Latent heat flux (W m^{-2})

       
        !+++CHECK+++
        ! Store the proxy for unstressed ecosystem functioning. This could be gpp, transpiration,
        ! evaporation. They all differ conceptually and numerically but the idea is the same.
        ! In the initial approcah we tried to calculate the stress as a ratio of the initial 
        ! transpiration over water supplied through the plant.  This approach resulted
        ! in inconsistencies likely because the many feedbacks in the calculation of transpir. 
        ! Similar issues were observed when using transpot. For this reason it was decided to drop
        ! this approach and use the ratio between the initial and adjusted gpp instead. After
        ! all this ratio will be used in stomate to adjust the allocation.
        ! Note that the feedback on gpp is calculated based on the ratio between ::transpir_supply 
        ! and transpir at the halmf hourly time step. In stomate we use the ratio between stressed
        ! and unstressed but at the daily time step. Night time values should be ignored in the
        ! daily calculation. If night time values are considered to unstressed this will skew
        ! the result.
        unstressed(:,:) = gpp(:,:)
        !+++++++++++


        IF (control_in%ok_hydrol_arch) THEN

           CALL hydrol_arch (kjpindex, circ_class_biomass, circ_class_n, temp_sol_new, &
                temp_air, swc, transpir_supply, dtradia, njsc, biomass, ind, soiltile, &
                ! vir_transpir_supply, veget, h_array_out, h_array_out, z_array_out3, &
                vir_transpir_supply, veget, h_array_out, z_array_out, &
                transpir_supply_column)

           ! transpir_supply is provided in mm per timestep (dtradia)
           hydrol_flag2(:) = .FALSE.
           hydrol_flag3(:,:) = .FALSE.
           transpir_init(:,:) = zero
 
           DO ji = 1, kjpindex

              !don't restrict gs during night
              !WRITE(numout,*) 'sinang, is zero when night',sinang(ji)
              DO jv = 1, nvm
                 IF (sinang(ji) .LT. min_sechiba) THEN
                    ! Night time we will never recalculate the energy budget
                    hydrol_flag2(ji) = .FALSE.
                    hydrol_flag3(ji,jv) = .FALSE.
                    transpir_init(ji, jv)= zero
                 ELSE
                    ! The ratio between ::transpir_supply and ::transpir is used to decide
                    ! whether we will recalculate gpp, transpir, ... (thus enerbil). This
                    ! is done every half hour. The waterstress that will be passed to stomate
                    ! to adjust the allocation will be calculated at the daily time step.
                    ! It is important NOT to include night time values in the daily mean
                    ! because more southern sides have shorter days during the growing season
                    ! the number of half hours with day light biases the waterstress if it 
                    ! is assumed that there is no waterstress at night.

                    IF (veget_max(ji,jv) .GT. zero .AND. &
                         veget(ji,jv) .GT. zero) THEN  
                        transpir_init(ji, jv)= (transpir(ji,jv)/veget_max(ji,jv)) 
                       !comparing the transpiration in tree-stand scale   
                       IF ( transpir_supply(ji,jv) .LT. transpir_init(ji,jv) ) THEN 
                          IF(ld_enerbil .OR. ld_wstress) WRITE(numout,*) '091013 hydrol_flag2&3 set to TRUE, pft is: ', jv
                          hydrol_flag2(ji) = .TRUE.
                          hydrol_flag3(ji,jv) = .TRUE.
                       ELSE
                          IF(ld_enerbil .OR. ld_wstress) WRITE(numout,*) '091013 hydrol_flag2 set to FALSE, pft is: ', jv
                       END IF
                    ELSE
                       ! There is no vegetation here, so we can skip it.
                       ! the flag at the pixel level (::hydrol_flag2) should 
                       ! not change its value.
                       hydrol_flag3(ji,jv) = .FALSE.
                       transpir_init(ji, jv)= zero
                    ENDIF
                    
                    !---TEMP---
                    IF(ld_wstress .AND. (jv.EQ.test_pft)) THEN
                       WRITE(numout,*) 'YC transpir_init:',   transpir_init(ji, jv)
                       WRITE(numout,*) 'transpir_supply, ',transpir_supply(ji,test_pft)
                       IF (veget_max(ji,test_pft) .GT. zero) THEN  
                          WRITE(numout,*) 'transpir modified, ', transpir(ji,test_pft)/veget_max(ji,test_pft) 
                       ENDIF
                    ENDIF
                    !----------

                 END IF

              END DO ! jv = 1, nvm

           END DO ! ji = 1, kjpindex
        
           ! now we call the module 'jdriver after transpir_supply', if there are grid squares
           !   for which the condition is satisfied
           
           hydrol_flag = .FALSE.

           DO ji = 1, kjpindex  ! grid point cycle

              IF  (hydrol_flag2(ji)) THEN    ! this switch is 'tripped' if the above applies to any PFT at the
                 ! grid point, as the whole energy budget for that grid point must
                 ! then be recalculated
                 hydrol_flag = .TRUE.
              ELSE
                 ! move on to the next PFT for the grid point in question
              END IF
              
           END DO ! ji = 1, kjpindex
        
           ! to switch off the enerbil/hydrol interface it would be necessary to set here: hydrol_flag = .FALSE.
           ! This is now externalized, so you can control it from the input file by specifying
           ! SECHIBA_GS_FEEDBACK = n
           IF(.NOT. control_in%ok_gs_feedback) hydrol_flag=.FALSE.

           IF (hydrol_flag) THEN
           
              ! 011013 jdriver should be called here again (to recalculate the energy budget because of
              !     supply term constriction
           
              ! WRITE(numout,*) '100114 before copy from temp_sol_new_save, temp_sol_new(1) is: ', temp_sol_new(1)

              ! ---------------------------------------------------------------------------------------------------  
              ! before 2nd call to enerbil, recall initial values (the ones un-changed from the first enerbil call)
           
              evapot(:) = evapot_save(:)                   !! Soil Potential Evaporation (mm day^{-1})
              evapot_corr(:) = evapot_corr_save(:)         !! Soil Potential Evaporation Correction (mm day^{-1})
              temp_sol(:) = temp_sol_save(:)               !! Soil temperature (K)
              qsurf(:) = qsurf_save(:)                     !! Surface specific humidity (kg kg^{-1})
              fluxsens(:) = fluxsens_save(:)               !! Sensible heat flux (W m^{-2})
              fluxlat(:) = fluxlat_save(:)                 !! Latent heat flux (W m^{-2})
              tsol_rad(:) = tsol_rad_save(:)               !! Tsol_rad (W m^{-2})
              vevapp(:) = vevapp_save(:)                   !! Total of evaporation (mm day^{-1})
              gpp(:,:) = gpp_save(:,:)                     !! Assimilation ((gC m^{-2}), total area)
              temp_sol_new(:) = temp_sol_new_save(:)       !! New soil temperature (K)
              vbeta2sum(:) = vbeta2sum_save(:) 
              vbeta3sum(:) = vbeta3sum_save(:)   
              ! ---------------------------------------------------------------------------------------------------
           
              DO ji = 1, kjpindex  ! grid point cycle
                 
                 ! 201013 calculate new series of beta_v3
              
                 IF (hydrol_flag) THEN  ! overall hydrol flag
                    
                    vbeta3_oldvalue(:, :) = 0.0d0
                 
                    ! vbeta2sum(:) = zero
                    IF (hydrol_flag2(ji)) THEN
                       DO jv = 1, nvm
                          IF (.NOT. hydrol_flag3(ji, jv)) THEN 
                             ! this is for cases when the supply term is higher than the 
                             !     transpiration) - i.e. the normal state of affairs
                          ELSE
                             vbeta3_oldvalue(ji, jv) = vbeta3(ji, jv)
                             ! Correction for the vbeta3 by replacing the "veget_max" to "veget"
                             !vbeta3(ji,jv) = (transpir_supply(ji,jv) * veget_max(ji,jv) ) / &
                             !     & ( dtradia * (1.0d0 - vbeta1(ji)) * (qsol_sat_new_out(ji) - qair_new(ji)) * &
                             !     & ( rau(ji) * (MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))) * tq_cdrag(ji)) )
                             ! vbeta3(ji,jv) = (transpir_supply(ji,jv) * veget(ji,jv) ) / &
                             !     & ( dtradia * (1.0d0 - vbeta1(ji)) * (qsol_sat_new_out(ji) - qair_new(ji)) * &
                             !      & ( rau(ji) * (MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))) * tq_cdrag(ji)) )
                             
                             !+++++++++CHECK++++++++++++++
                             ! We calucalate the vbeta3 based on the ::transpir_supply term for each grid and pft
                             ! We should keep the spatial weighting fraction,::veget_max, for vebta3. Even the calculation
                             ! of ::transpir_supply is in tree stand scale.   
                             IF (veget_max(ji,jv) .GT. zero .AND. veget(ji,jv) .GT. zero) THEN        
                              vbeta3(ji,jv) = veget_max(ji, jv) * transpir_supply(ji,jv)  / &  
                                   &  ( dtradia * (1.0d0 - vbeta1(ji)) * (qsol_sat_new_out(ji) - qair_new(ji)) * &
                                   &  ( rau(ji) * (MAX(min_wind, SQRT (u(ji)*u(ji) + v(ji)*v(ji)))) * tq_cdrag(ji)) )
                             ELSE
                               !No vegetation presenst for this condition, then set vbeta3 equal to zero. 
                               vbeta3(ji,jv) = 0.0d0
                             ENDIF
                             !++++++++++++++++++++++++++++ 
                             !IF (jv == test_pft .AND. test_grid == 1) THEN    
                             !   WRITE(numout, '(A40)'        ) '=======Water Stress On======'
                             !   WRITE(numout, '(A20, I14)'   ) '=======PFT======:', jv
                             !   WRITE(numout, '(A20, ES14.4)') 'Old_Vbeta3_value:', vbeta3_oldvalue(ji, jv)
                             !   WRITE(numout, '(A20, ES14.4)') 'New_Vbeta3_value:', vbeta3(ji, jv)
                             !ENDIF      
 
                              IF ( ( vbeta3(ji, jv)/vbeta3_oldvalue(ji, jv) ) .gt. 1.0d0 ) THEN
                                ! WRITE(numout,*) 'vbeta3 ratio is greater than 1!!'
                             END IF
                          END IF ! (hydrol_flag3(ji, jv) .eq. .FALSE.)
                       ENDDO ! jv = 1, nvm
                    ELSE
                       ! do nothing
                    END IF ! (hydrol_flag2(ji) .eq. .TRUE.)        
                 END IF ! IF (hydrol_flag .eq. .TRUE.)  
                 
              END DO ! ji = 1, kjpindex
              
              ! now to approximate the stomatatal conductance, where it is constrained by the supply term.
              ! This calculation takes place over the whole canopy, for now.

              ! To calculate the gs_effective (the effective stomatal conductance), first we need to calculate
              !   a constrained vbeta3, and work back from there, using a tweaked version of diffuco_trans_co2

              IF (ld_jameswrite) THEN
                  WRITE(numout, *) '020315b in sechiba, SUM(laieff_isotrop(1:jnlvls_canopy, 1, 4) is: ', SUM(laieff_isotrop(1:jnlvls_canopy, 1, 4))
                  WRITE(numout, *) '130315 before jdriver'
              END IF !(ld_jameswrite)

              CALL jdriver (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, &
                   & index, indexveg, zlev, lwdown, swnet, swdown, epot_air, temp_air, &
                   & u, v, petAcoef, petBcoef, qair, peqAcoef, peqBcoef, pb, rau, &
                   & vbeta, valpha, vbeta1, vbeta2, vbeta3, vbeta3pot, vbeta4, vbeta5, &
                   & emis, soilflx, soilcap, tq_cdrag, humrel, &
!!$                   & cimean, ccanopy, emis, soilflx, soilcap, tq_cdrag, humrel, &
                   & fluxsens, fluxlat, vevapp, transpir, transpot, vevapnu, vevapwet, &
!!$                   & fluxsens, fluxlat, vevapp, transpir, transpot, gpp, vevapnu, vevapwet, &
                   & vevapsno, vevapflo, t2mdiag, temp_sol, tsol_rad, temp_sol_new, qsurf, &
                   & evapot, evapot_corr, rest_id, hist_id, hist2_id, & 
                   & flux_ground_h, flux_ground_le, flux_h_grid, flux_le_grid, t_a_next_grid, q_a_next_grid, & 
                   & temp_surf_next, temp_atmos_pres_grid, q_atmos_pres_grid, &
                   & james_netcdf_flag, james_step_count, temp_leaf_pres_grid, temp_surf_pres, &
                   & transpir_supply, hydrol_flag, hydrol_flag2, &
                   & hydrol_flag3, vbeta2sum, vbeta3sum, veget_max, qsol_sat_new_out, qair_new, &
                   & delta_temp_save, netrad_out, veget, Collim_Abs_Tot, Collim_Alb_Tot, &
                   & laieff_isotrop, h_array_out, z_array_out, transpir_supply_column, &
                   & u_speed, profile_vbeta3, profile_rveget, max_height_store, &
                   & u_speed_grid, flux_rn_grid, t2m_month_out )
              
              ! we need to pass on the new vbeta3 to jdiffuco_trans_co2
                         
             CALL jdiffuco_trans_co2 (kjpindex, dtradia, swdown, temp_sol, pb, qsurf, &
                   & q2m, t2m, temp_growth, rau, u, v, tq_cdrag, vegstress, &
                   & assim_param, ccanopy, veget_max, lai, qsintveg, qsintmax, &
                   & vbeta3, vbeta3pot, rveget, rstruct, cimean, gsmean, gpp, &
                   & vbeta23, Isotrop_Abs_Tot_p, Isotrop_Tran_Tot_p, lai_per_level, &
                   & leaf_ci, hydrol_flag2, hydrol_flag3, gs_distribution, gs_diffuco_output, &
                   & james_netcdf_flag, gstot_component, gstot_frac, warnings, veget, biomass, &
                   & u_speed, profile_vbeta3, profile_rveget)
             

              DO ji = 1, kjpindex

                 DO  jv = 1, nvm

                    IF (hydrol_flag3(ji,jv)) THEN

                       ! do nothing (as the relevant values were calculated in jdiffuco_trans_co2)
                       
                    ELSE
                       ! ------------------------------------------------------------------------------------------
                       ! restore saved values for PFTs and grid squares not affected by water stress

                       vbeta3(ji,jv) = vbeta3_save(ji,jv)        !! Beta for Transpiration (unitless)
                       vbeta3pot(ji,jv) = vbeta3pot_save(ji,jv)  !! Beta for Potential Transpiration
                       rveget(ji,jv) = rveget_save(ji,jv)        !! stomatal resistance of vegetation 
                       rstruct(ji,jv) = rstruct_save(ji,jv)      !! structural resistance @tex ($s m^{-1}$) @endtex
                       cimean(ji,jv) = cimean_save(ji,jv)        ! mean intercellular CO2 concentration
                       gsmean(ji,jv) = gsmean_save(ji,jv)        ! mean stomatal conductance to CO2 (umol m-2 s-1)
                       gpp(ji,jv) = gpp_save(ji,jv)              !! gross primary production
                       ! ------------------------------------------------------------------------------------------

                    END IF

                 END DO ! jv = 1, nvm
                 
              END DO ! ji = 1, kjpindex

           ELSE
              ! do nothing
           END IF

           !+++CHECK+++
           ! Store the proxy for stressed ecosystem functioning. This value is used in stomate
           ! to calculate the waterstress used in allocation
           stressed(:,:) = gpp(:,:)
           !+++++++++++
           
        END IF   ! IF (control_in%ok_hydrol_arch) THEN

        ! %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

        james_count = james_count + 1

      
        !! 2.3.1 Write some data to the history files.
!!$        CALL enerbil_write (kjit, kjpindex, index, lwdown, ccanopy, temp_sol, &
        CALL enerbil_write (kjit, kjpindex, index, lwdown, temp_sol, &
            & temp_sol_new, evapot, evapot_corr, hist_id, hist2_id)


     ELSE
    
        !! 2.3.1 Compute energy balance using standard method if control%ok_enerbil is FALSE
        !!       Second calling "enerbil_main" using standard method and and write varibles 
        !!       out to history file 
        !!	 write(numout, *) "YC: Compute energy balance using standard method
        CALL enerbil_main (kjit, kjpindex, dtradia, ldrestart_read, myfalse, &
            & index, indexveg, zlev, lwdown, swnet, swdown, &
            & epot_air, temp_air, u, v, petAcoef, &
            & petBcoef, qair, peqAcoef, peqBcoef, pb, &
            & rau, vbeta, valpha, vbeta1, vbeta2, &
            & vbeta3, vbeta3pot, vbeta4, vbeta5, emis, &
            & soilflx, soilcap, tq_cdrag, humrel, fluxsens, &
            & fluxlat, vevapp, transpir, transpot, vevapnu, &
            & vevapwet, vevapsno, vevapflo, t2mdiag, temp_sol, &
            & tsol_rad, temp_sol_new, qsurf, evapot, evapot_corr, &
            & rest_id, hist_id, hist2_id, beta_diffuco, &
            & precip_rain, snow, sfcfrz, radsink, &
            & veget_max, osfcmelt, pgflux, gthick, gtemp, &
            & gpkappa, grndflux, albedo_undersnow, snowtemp, snowdz, &
            & radxs, pkappa_snow, precip_snow, snowliq, snowheat, & 
            & snowrho, snowgrain, snowdz_o, soilflxresid)

    END IF ! (control%ok_nenerbil)
    
    soilflx_out=soilflx !add by YC 20140313 copy the soilflx to soilflx_out

    !! 2.4 Compute hydrology
    IF ( .NOT. hydrol_cwrr ) THEN
       ! 2.4.1 Water balance from Choisnel module (2 soil layers)
       CALL hydrolc_main (kjit, kjpindex, dtradia, ldrestart_read, myfalse, &
               & index, indexveg, control_in, temp_sol_new, floodout, &
               & runoff, drainage, frac_nobio, totfrac_nobio, vevapwet, &
               & veget, veget_max, qsintmax, qsintveg, vevapnu, &
               & vevapsno, vevapflo, snow, snow_age, snow_nobio, &
               & snow_nobio_age, tot_melt, transpir, precip_rain, precip_snow, &
               & returnflow, reinfiltration, irrigation, humrel, vegstress, &
               & rsol, drysoil_frac, evapot, evapot_corr, flood_frac, &
               & flood_res, shumdiag, litterhumdiag, soilcap, rest_id, &
               & hist_id, hist2_id, temp_air, pb, u, v, &
               & swnet, albedo_undersnow, pgflux, snowrho, snowtemp, &
               & soiltemp, snowgrain, snowdz, snowheat, snowliq, &
               & sfcfrz, osfcmelt, grndflux, radsink, radxs, &
               & gtemp, soilflxresid)

       evap_bare_lim(:) = -un
       k_litt(:) = huit

       ! No specific calculation for shumdiag_perma. We assume it to shumdiag. 
       shumdiag_perma(:,:)=shumdiag(:,:) 
    ELSE
       !! 2.4.1 Water balance from CWRR module (11 soil layers)
       CALL hydrol_main (kjit, kjpindex, dtradia, ldrestart_read, myfalse, &
               & index, indexveg, indexsoil, indexlayer, indexnbdl, &
               & control_in, temp_sol_new, floodout, runoff, drainage, &
               & frac_nobio, totfrac_nobio, vevapwet, veget, veget_max, &
               & njsc, qsintmax, qsintveg, vevapnu, vevapsno, &
               & vevapflo, snow, snow_age, snow_nobio, snow_nobio_age, &
               & tot_melt, transpir, precip_rain, precip_snow, returnflow, &
               & reinfiltration, irrigation, humrel, vegstress, drysoil_frac, &
               & evapot, evapot_corr, evap_bare_lim, flood_frac, flood_res, &
               & shumdiag, shumdiag_perma, k_litt, litterhumdiag, soilcap, &
               & soiltile, reinf_slope, rest_id, hist_id, hist2_id, &
               & swc, veg_soiltext, stempdiag, temp_air, pb, &
               & u, v, swnet, albedo_undersnow, pgflux, &
               & snowrho, snowtemp, soiltemp, snowgrain, snowdz, &
               & snowheat, snowliq, sfcfrz, grndflux, radsink, &
               & radxs, gtemp, soilflxresid)

       rsol(:) = -un

    ENDIF

    IF ( .NOT. control%ok_nenerbil) THEN
        IF (control_in%ok_hydrol_arch) THEN
            !!+++CHECK: move to enerbill?+++
            CALL hydrol_arch (kjpindex, circ_class_biomass, circ_class_n, temp_sol_new, &
                 temp_air, swc, transpir_supply, dtradia, njsc, biomass, ind, soiltile, &
                 ! vir_transpir_supply, veget, h_array_out, &
                 vir_transpir_supply, veget, &
                 h_array_out, z_array_out, transpir_supply_column)
            !!+++++++++++

        ENDIF
    END IF ! ( .NOT. control%ok_nenerbil)
  
    !! 2.5 Compute remaining components of the energy balance
    CALL enerbil_fusion (kjpindex, dtradia, tot_melt, soilcap, snowdz_o, & 
         temp_sol_new, fusion)
   
    !! 2.6 Compute surface variables (emissivity, albedo and roughness)
    CALL condveg_main (kjit, kjpindex, dtradia, ldrestart_read, &
         ldrestart_write, index, lalo, neighbours, resolution, contfrac, &
         veget, veget_max, frac_nobio, totfrac_nobio, zlev, &
         snow, snow_age, snow_nobio, snow_nobio_age, drysoil_frac, &
         height,  emis, albedo, z0, roughheight, &
         rest_id, hist_id, hist2_id, lai,  sinang, &
         circ_class_biomass, circ_class_n, lai_split, z0_veg, Isotrop_Abs_Tot_p, &
         Isotrop_Tran_Tot_p, laieff_fit, albedo_pft, frac_snow_nobio, frac_snow_veg, &
         albedo_undersnow, snowdz, snowrho, snowliq, precip_snow, &
         osfcmelt, temp_air, u, v, &
         Collim_Abs_Tot, Collim_Alb_Tot, laieff_collim, laieff_isotrop)

    !! 2.7 Compute soil thermodynamics
    CALL thermosoil_main (kjit, kjpindex, dtradia, ldrestart_read, & 
 	& myfalse, index, indexgrnd,indexnbdl, & 
 	& control_in,temp_sol_new, snow, soilcap, soilflx, shumdiag_perma, stempdiag, & 
 	& ptnlev1, rest_id, hist_id, hist2_id, & 
 	& soiltemp,pb,grndflux,snowrho,snowdz,snowtemp,gthick,gtemp,gpkappa,& 
 	& pkappa_snow,snowdz_o) 

    !! 2.8 Compute river routing 
    IF ( river_routing .AND. nbp_glo .GT. 1) THEN
       !! 2.8.1 River routing
       CALL routing_main (kjit, kjpindex, dtradia, control_in, ldrestart_read, myfalse, index, &
            & lalo, neighbours, resolution, contfrac, totfrac_nobio, veget_max, floodout, runoff, &
            & drainage, transpot, precip_rain, humrel, k_litt, flood_frac, flood_res, &
            & stempdiag, reinf_slope, returnflow, reinfiltration, irrigation, riverflow, coastalflow, rest_id, hist_id, hist2_id)
    ELSE
       !! 2.8.2 No routing, set variables to zero
       riverflow(:) = zero
       coastalflow(:) = zero
       returnflow(:) = zero
       reinfiltration(:) = zero
       irrigation(:) = zero
       flood_frac(:) = zero
       flood_res(:) = zero
    ENDIF



    !! 2.9 Compute slow processes (i.e. 'daily' and annual time step)
    ! ::ok_co2 and ::ok_stomate are flags that determine whether the
    ! forcing files are written.
    CALL slowproc_main (kjit, kjpij, kjpindex, dtradia, date0, &
         ldrestart_read, myfalse, control_in%ok_co2, control_in%ok_stomate, index, &
         indexveg, lalo, neighbours, resolution, contfrac, &
         soiltile, reinf_slope, t2mdiag, t2mdiag, temp_sol, &
         stempdiag, vegstress, shumdiag, litterhumdiag, precip_rain, &
         precip_snow, gpp, deadleaf_cover, assim_param, lai, &
         frac_age, height, veget, frac_nobio, njsc, &
         veget_max, totfrac_nobio, qsintmax, rest_id, hist_id, &
         hist2_id, rest_id_stom, hist_id_stom, hist_id_stom_IPCC, co2_flux, &
         fco2_lu, sinang, circ_class_biomass, circ_class_n, &
         transpir, stressed, unstressed, transpir_supply, &
         biomass, ind, lai_per_level, laieff_fit, &
         vir_transpir_supply, temp_growth, Isotrop_Tran_Tot_p,&
         sum_veget_diff, snow, jnlevels_loc, h_array_out, z_array_out, &
         max_height_store, t2m_month_out)

    !! 2.9 Compute global CO2 flux
    !+++CHECK+++
    ! If co2flux contains soil respiration the loop should include the bare soil
    ! because after a land cover change PFT1 can contain litter and soil carbon
    netco2flux(:) = zero
    DO jv = 2,nvm
       netco2flux(:) = netco2flux(:) + co2_flux(:,jv)*veget_max(:,jv)
    ENDDO
    !+++++++++++

    !! 2.10 Swap old for newly computed values
    !! 2.10 Update the temperature (temp_sol) with newly computed values 
    CALL sechiba_end (kjpindex, dtradia, temp_sol_new, snowtemp, snowdz_o, & 
         temp_sol)

    !! 2.11 Write internal variables to output fields
    z0_out(:) = z0(:)
    emis_out(:) = emis(:)
    albedo_out(:,:) = albedo(:,:) 
    qsurf_out(:) = qsurf(:)
    nlevelsloc_out(:) = jnlevels_loc(:)
 
    !! 2.12 Write global variables to history files
    sum_treefrac(:) = zero
    sum_grassfrac(:) = zero
    sum_cropfrac(:) = zero
    DO jv = 2, nvm 
       IF (is_tree(jv) .AND. natural(jv)) THEN
          sum_treefrac(:) = sum_treefrac(:) + veget_max(:,jv)
       ELSE IF ((.NOT. is_tree(jv))  .AND. natural(jv)) THEN
          sum_grassfrac(:) = sum_grassfrac(:) + veget_max(:,jv)
       ELSE 
          sum_cropfrac = sum_cropfrac(:) + veget_max(:,jv)
       ENDIF
    ENDDO

    ! This is to print the tree density in each circumference class to the history file.
    ! Originally this was just temporary, but it may serve a good purpose.
    DO icir=1,ncirc
       WRITE (var_name,"('CCN_',i3.3)") icir
       CALL histwrite_p(hist_id_stom, var_name, kjit, circ_class_n(:,:,icir), kjpindex*nvm, indexveg)
    ENDDO

    CALL xios_orchidee_send_field("evapnu",vevapnu*one_day/dt_sechiba)
    CALL xios_orchidee_send_field("snow",snow)
    CALL xios_orchidee_send_field("snowage",snow_age)
    CALL xios_orchidee_send_field("snownobio",snow_nobio)
    CALL xios_orchidee_send_field("snownobioage",snow_nobio_age)
    CALL xios_orchidee_send_field("reinf_slope",reinf_slope)
    CALL xios_orchidee_send_field("soilindex",REAL(njsc, r_std))
    CALL xios_orchidee_send_field("vegetfrac",veget)
    CALL xios_orchidee_send_field("maxvegetfrac",veget_max)
    !CALL xios_orchidee_send_field("nobiofrac",frac_nobio)
    CALL xios_orchidee_send_field("soiltile",soiltile)
    CALL xios_orchidee_send_field("rstruct",rstruct)
    CALL xios_orchidee_send_field("gpp",gpp/dt_sechiba)
    CALL xios_orchidee_send_field("nee",co2_flux/dt_sechiba)
    CALL xios_orchidee_send_field("drysoil_frac",drysoil_frac)
    CALL xios_orchidee_send_field("evapflo",vevapflo*one_day/dt_sechiba)
    CALL xios_orchidee_send_field("evapflo_alma",vevapflo/dt_sechiba)
    CALL xios_orchidee_send_field("k_litt",k_litt)
    CALL xios_orchidee_send_field("beta",vbeta)
    CALL xios_orchidee_send_field("vbeta1",vbeta1)
    CALL xios_orchidee_send_field("vbeta2",vbeta2)
    CALL xios_orchidee_send_field("vbeta3",vbeta3)
    CALL xios_orchidee_send_field("vbeta4",vbeta4)
    CALL xios_orchidee_send_field("vbeta5",vbeta5)
!    CALL xios_orchidee_send_field("vbetaco2",vbetaco2)
    CALL xios_orchidee_send_field("gsmean",gsmean) 
    CALL xios_orchidee_send_field("cimean",cimean)
    CALL xios_orchidee_send_field("rveget",rveget)
    CALL xios_orchidee_send_field("rsol",rsol)

! Note that 0.0005555556 is used instead of one_day/dt temporary to have the same 
! order of cacluation as using IOIPSL for compairing of resluts. This will be changed in futur update.   
    histvar(:)=SUM(vevapwet(:,:),dim=2)
    CALL xios_orchidee_send_field("evspsblveg",histvar/dt_sechiba)
    histvar(:)= vevapnu(:)+vevapsno(:)
    CALL xios_orchidee_send_field("evspsblsoi",histvar/dt_sechiba)
    histvar(:)=SUM(transpir(:,:),dim=2)
    CALL xios_orchidee_send_field("tran",histvar/dt_sechiba)
    !histvar(:)= sum_treefrac(:)*100*contfrac(:)
    !CALL xios_orchidee_send_field("treeFrac",histvar)
    !histvar(:)= sum_grassfrac(:)*100*contfrac(:)
    !CALL xios_orchidee_send_field("grassFrac",histvar)
    !histvar(:)= sum_cropfrac(:)*100*contfrac(:)
    !CALL xios_orchidee_send_field("cropFrac",histvar)
    !histvar(:)=veget_max(:,1)*100*contfrac(:)
    !CALL xios_orchidee_send_field("baresoilFrac",histvar)
    !histvar(:)=SUM(frac_nobio(:,1:nnobio),dim=2)*100*contfrac(:)
    !CALL xios_orchidee_send_field("residualFrac",histvar)

    CALL xios_orchidee_send_field("tsol_rad",tsol_rad-273.15)
    CALL xios_orchidee_send_field("qsurf",qsurf)
    CALL xios_orchidee_send_field("albedo",albedo)
    CALL xios_orchidee_send_field("albedo_pft",albedo_pft)
    CALL xios_orchidee_send_field("emis",emis)
    CALL xios_orchidee_send_field("z0",z0)
    CALL xios_orchidee_send_field("roughheight",roughheight)
    CALL xios_orchidee_send_field("lai",lai)
    CALL xios_orchidee_send_field("subli",vevapsno*one_day/dt_sechiba)
    CALL xios_orchidee_send_field("vevapnu",vevapnu*one_day/dt_sechiba)
    CALL xios_orchidee_send_field("vevapnu_alma",vevapnu/dt_sechiba)
    CALL xios_orchidee_send_field("transpir",transpir*one_day/dt_sechiba)
    CALL xios_orchidee_send_field("transpot",transpot*one_day/dt_sechiba)
    CALL xios_orchidee_send_field("inter",vevapwet*one_day/dt_sechiba)
    CALL xios_orchidee_send_field("Qf",fusion)
    
    CALL xios_orchidee_send_field("fluxnet",netrad_out)    ! 20140313 Add by YC
    CALL xios_orchidee_send_field("fluxsoil", soilflx_out) ! 20140313 Add by YC
         !CALL xios_orchidee_send_field("LAIPIX", laipix)        !20140521 Add by YC
    CALL xios_orchidee_send_field("swe",snow)    
         
    histvar(:)=zero
    DO jv=1,nvm
      histvar(:) = histvar(:) + vevapwet(:,jv)
    ENDDO
    CALL xios_orchidee_send_field("ECanop",histvar/dt_sechiba)
    histvar(:)=zero
    DO jv=1,nvm
      histvar(:) = histvar(:) + transpir(:,jv)
    ENDDO
    CALL xios_orchidee_send_field("TVeg",histvar/dt_sechiba)
    CALL xios_orchidee_send_field("ACond",tq_cdrag)

    IF ( .NOT. almaoutput ) THEN
       ! Write history file in IPSL-format
       CALL histwrite_p(hist_id, 'beta', kjit, vbeta, kjpindex, index)
       CALL histwrite_p(hist_id, 'z0', kjit, z0, kjpindex, index)
       CALL histwrite_p(hist_id, 'roughheight', kjit, roughheight, kjpindex, index)
       CALL histwrite_p(hist_id, 'vegetfrac', kjit, veget, kjpindex*nvm, indexveg)
      ! laipix(:) = zero
       laieff_pix (:) = zero
       laieff_isotrop (:,:,:) = zero
       laieff_pft (:,:) = zero
      ! lai(:,1) = zero
      ! laipix(:) = SUM(lai(:,:)*veget_max(:,:),2)
      ! !WRITE (numout,*) 'lai'
      ! WRITE (numout,*) 'lai per pixel', laipix 
      ! CALL histwrite_p(hist_id_stom, 'laipix', kjit, laipix, kjpindex, index)
       isotrop_angle=COS(60.0/180.0*pi)

       DO ipts=1,kjpindex
          DO ivm=1,nvm
             DO ilevel=1,nlevels_tot
                laieff_isotrop(ilevel,ipts,ivm)=calculate_laieff_fit(isotrop_angle,laieff_fit(ipts,ivm,ilevel))
                ! Because we are fitting these values, it's possible that some will 
                ! drop below zero.  This causes a serious problem here,
                ! so let's make sure this doesn't happen.  Unfortunately, by
                ! doing this, we lose a test of things going wrong (an unfitted
                ! negative LAIeff can be the sign of another problem), but we
                ! don't really have a choice.
                IF(laieff_isotrop(ilevel,ipts,ivm) .LT. min_stomate) &
                     laieff_isotrop(ilevel,ipts,ivm)=zero
             ENDDO
          ENDDO
       ENDDO

       laieff_pft(:,:) = SUM(laieff_isotrop(:,:,:),1)
       laieff_pix(:)=SUM(laieff_pft(:,:)*veget_max(:,:),2)

       CALL histwrite_p(hist_id, 'laieffpix',kjit,laieff_pix,kjpindex, index)

       CALL histwrite_p(hist_id, 'maxvegetfrac', kjit, veget_max, kjpindex*nvm, indexveg)     
       !CALL histwrite_p(hist_id, 'nobiofrac', kjit, frac_nobio, kjpindex*nnobio, indexnobio)
       CALL histwrite_p(hist_id, 'lai', kjit, lai, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'subli', kjit, vevapsno, kjpindex, index)
       CALL histwrite_p(hist_id, 'evapnu', kjit, vevapnu, kjpindex, index)
       CALL histwrite_p(hist_id, 'transpir', kjit, transpir, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'transpot', kjit, transpot, kjpindex*nvm, indexveg)
       IF (control_in%ok_hydrol_arch) THEN
          CALL histwrite_p(hist_id, 'transpir_supply', kjit, transpir_supply, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist_id, 'transpir_supplysec', kjit, transpir_supply, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist_id, 'transpir_init', kjit, transpir_init, kjpindex*nvm, indexveg)
       ENDIF

       CALL histwrite_p(hist_id, 'inter', kjit, vevapwet, kjpindex*nvm, indexveg)

       CALL histwrite_p(hist_id, 'vbeta1', kjit, vbeta1, kjpindex, index)
       CALL histwrite_p(hist_id, 'vbeta2', kjit, vbeta2, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'vbeta3', kjit, vbeta3, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'vbeta3pot', kjit, vbeta3pot, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'vbeta4', kjit, vbeta4, kjpindex, index)    
       CALL histwrite_p(hist_id, 'vbeta5', kjit, vbeta5, kjpindex, index)
    
       CALL histwrite_p(hist_id, 'drysoil_frac', kjit, drysoil_frac, kjpindex, index)
       CALL histwrite_p(hist_id, 'rveget', kjit, rveget, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'rstruct', kjit, rstruct, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'photo_warn', kjit, warnings(:,:,iwphoto), kjpindex*nvm, indexveg)
       !+++++++++++++++++++++++++++++++++++++++++++++++++++
       ! Adde by YC 20140313  For Checking Energy Budget    
       CALL histwrite_p(hist_id, 'fluxnet',  kjit, netrad_out,  kjpindex, index) 
       CALL histwrite_p(hist_id, 'fluxsoil', kjit, soilflx_out, kjpindex, index) 
       !+++++++++++++++++++++++++++++++++++++++++++++++++++ 
       CALL histwrite_p(hist_id, 'swe',      kjit, snow,        kjpindex, index)
              
       IF ( .NOT. control_in%hydrol_cwrr ) THEN
          CALL histwrite_p(hist_id, 'rsol', kjit, rsol, kjpindex, index)
       ENDIF

       ! CALL histwrite_p(hist_id, 'snow', kjit, snow, kjpindex, index)
       ! CALL histwrite_p(hist_id, 'snowage', kjit, snow_age, kjpindex, index)
       ! CALL histwrite_p(hist_id, 'snownobio', kjit, snow_nobio, kjpindex*nnobio, indexnobio)
       ! CALL histwrite_p(hist_id, 'snownobioage', kjit, snow_nobio_age, kjpindex*nnobio, indexnobio)

       IF (ok_explicitsnow) THEN
          CALL histwrite_p(hist_id, 'grndflux', kjit, grndflux, kjpindex,index) 
          CALL histwrite_p(hist_id, 'sfcfrz', kjit, sfcfrz, kjpindex, index) 	 
          CALL histwrite_p(hist_id, 'snowtemp',kjit,snowtemp,kjpindex*nsnow,indexsnow) 
          CALL histwrite_p(hist_id, 'soiltemp',kjit,soiltemp,kjpindex*ngrnd,indexgrnd) 
          CALL histwrite_p(hist_id, 'snowliq', kjit,snowliq,kjpindex*nsnow,indexsnow) 
          CALL histwrite_p(hist_id, 'snowdz', kjit,snowdz,kjpindex*nsnow,indexsnow) 
          CALL histwrite_p(hist_id, 'snowrho', kjit,snowrho,kjpindex*nsnow,indexsnow) 
          CALL histwrite_p(hist_id, 'snowheat',kjit,snowheat,kjpindex*nsnow,indexsnow) 
          CALL histwrite_p(hist_id, 'snowgrain',kjit,snowgrain,kjpindex*nsnow,indexsnow) 
          CALL histwrite_p(hist_id, 'radsink',kjit,radsink,kjpindex*nsnow,indexsnow)
          CALL histwrite_p(hist_id, 'frac_snow_veg',kjit,frac_snow_veg,kjpindex,index)  
          CALL histwrite_p(hist_id, 'frac_snow_nobio', kjit,frac_snow_nobio,kjpindex*nnobio, indexnobio)
          CALL histwrite_p(hist_id, 'pgflux',kjit,pgflux,kjpindex,index)
          CALL histwrite_p(hist_id, 'soiltile',  kjit, soiltile, kjpindex*nstm, indexsoil)
       END IF 

      !
       IF ( control_in%hydrol_cwrr ) THEN
          CALL histwrite_p(hist_id, 'soilindex',  kjit, REAL(njsc, r_std), kjpindex, index)
          CALL histwrite_p(hist_id, 'reinf_slope',  kjit, reinf_slope, kjpindex, index)
          CALL histwrite_p(hist_id, 'k_litt', kjit, k_litt, kjpindex, index)
       ENDIF
       IF ( control_in%do_floodplains ) THEN
          CALL histwrite_p(hist_id, 'evapflo', kjit, vevapflo, kjpindex, index)
          CALL histwrite_p(hist_id, 'flood_frac', kjit, flood_frac, kjpindex, index)
       ENDIF
       IF ( control_in%ok_co2 ) THEN
          CALL histwrite_p(hist_id, 'gsmean', kjit, gsmean, kjpindex*nvm, indexveg)    
          CALL histwrite_p(hist_id, 'gpp', kjit, gpp, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist_id, 'cimean', kjit, cimean, kjpindex*nvm, indexveg)    
       ENDIF
       IF ( control_in%ok_stomate ) THEN
          CALL histwrite_p(hist_id, 'nee', kjit, co2_flux, kjpindex*nvm, indexveg)    
       ENDIF
       IF ( control%ok_C13 ) THEN
          CALL histwrite_p(hist_id, 'delta_c13_assim', kjit, delta_c13_assim, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist_id, 'leaf_ci_out', kjit, leaf_ci_out, kjpindex*nvm, indexveg)  
       ENDIF
       
       ! This is a tricky variable.  It's related to the canopy structure
       ! and won't have a value if stomate is not activated.  However, it's
       ! updated at every sechiba timestep.  So I have to put it here.
       DO ivm=1,nvm
          WRITE(var_name,'(A,I3.3)') 'ABS_CANOPY_LIGHT_',ivm
          CALL histwrite_p (hist_id, var_name, kjit, &
               Isotrop_Abs_Tot_p(:,ivm,:), kjpindex*nlevels_tot, indexcan) 
       ENDDO

       histvar(:)=SUM(vevapwet(:,:),dim=2)
       CALL histwrite_p(hist_id, 'evspsblveg', kjit, histvar, kjpindex, index)

       histvar(:)= vevapnu(:)+vevapsno(:)
       CALL histwrite_p(hist_id, 'evspsblsoi', kjit, histvar, kjpindex, index)

       histvar(:)=SUM(transpir(:,:),dim=2)
       CALL histwrite_p(hist_id, 'tran', kjit, histvar, kjpindex, index)

       !histvar(:)= sum_treefrac(:)*100*contfrac(:)
       !CALL histwrite_p(hist_id, 'treeFrac', kjit, histvar, kjpindex, index) 

       !histvar(:)= sum_grassfrac(:)*100*contfrac(:)
       !CALL histwrite_p(hist_id, 'grassFrac', kjit, histvar, kjpindex, index) 

       !histvar(:)= sum_cropfrac(:)*100*contfrac(:)
       !CALL histwrite_p(hist_id, 'cropFrac', kjit, histvar, kjpindex, index)

       !histvar(:)=veget_max(:,1)*100*contfrac(:)
       !CALL histwrite_p(hist_id, 'baresoilFrac', kjit, histvar, kjpindex, index)

       !histvar(:)=SUM(frac_nobio(:,1:nnobio),dim=2)*100*contfrac(:)
       !CALL histwrite_p(hist_id, 'residualFrac', kjit, histvar, kjpindex, index)
       ! For the snow fraction, we combine the snow fraction calculated for the
       ! vegetated and non-vegetated fractions to get a value for the whole pixel.
       frac_snow_pix(:)=zero
       DO ivm=1,nvm
          frac_snow_pix(:)=frac_snow_pix(:)+frac_snow_veg(:,ivm)*veget_max(:,ivm)
       ENDDO
       DO inno=1,nnobio
          frac_snow_pix(:)=frac_snow_pix(:)+frac_snow_nobio(:,inno)*frac_nobio(:,inno)
       ENDDO
       CALL histwrite_p(hist_id, 'snowfrac', kjit, frac_snow_pix, kjpindex, index)
       !+++++ CHECK +++++++
       ! At one time we were having negative snow fractions.  I think that is
       ! fixed, but I'm leaving this check in here for a little longer.

       DO ipts=1,kjpindex
          IF(frac_snow_pix(ipts) .LT. 0.0)THEN
             WRITE(numout,*) 'SNOWFRAC ipts ',ipts,frac_snow_pix(ipts)
             DO ivm=1,nvm
                WRITE(numout,*) ivm,frac_snow_veg(ipts,ivm),veget_max(ipts,ivm)
             ENDDO
             WRITE(numout,*) 'NOBIO '
             DO inno=1,nnobio
                WRITE(numout,*) ivm,frac_snow_nobio(ipts,inno),frac_nobio(ipts,inno)
             ENDDO
             CALL ipslerr_p (3,'sechiba', 'snow frac error','','')
          ENDIF
       ENDDO

       !++++++++++++++++++

    ELSE
       ! Write history file in ALMA format 
       CALL histwrite_p(hist_id, 'vegetfrac', kjit, veget, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'maxvegetfrac', kjit, veget_max, kjpindex*nvm, indexveg)
       CALL histwrite_p(hist_id, 'nobiofrac', kjit, frac_nobio, kjpindex*nnobio, indexnobio)
       CALL histwrite_p(hist_id, 'Qf', kjit, fusion, kjpindex, index)
       CALL histwrite_p(hist_id, 'ESoil', kjit, vevapnu, kjpindex, index)
       CALL histwrite_p(hist_id, 'EWater', kjit, vevapflo, kjpindex, index)
       CALL histwrite_p(hist_id, 'SWE', kjit, snow, kjpindex, index)
       histvar(:)=zero
       DO jv=1,nvm
          histvar(:) = histvar(:) + transpir(:,jv)
       ENDDO
       CALL histwrite_p(hist_id, 'TVeg', kjit, histvar, kjpindex, index)
       histvar(:)=zero
       DO jv=1,nvm
          histvar(:) = histvar(:) + vevapwet(:,jv)
       ENDDO
       CALL histwrite_p(hist_id, 'ECanop', kjit, histvar, kjpindex, index)
       CALL histwrite_p(hist_id, 'ACond', kjit, tq_cdrag, kjpindex, index)
       !
       CALL histwrite_p(hist_id, 'Z0', kjit, z0, kjpindex, index)
       CALL histwrite_p(hist_id, 'EffectHeight', kjit, roughheight, kjpindex, index)
       !
       IF ( control_in%do_floodplains ) THEN
          CALL histwrite_p(hist_id, 'Qflood', kjit, vevapflo, kjpindex, index)
          CALL histwrite_p(hist_id, 'FloodFrac', kjit, flood_frac, kjpindex, index)
       ENDIF
       !
       IF ( control_in%ok_co2 ) THEN
          CALL histwrite_p(hist_id, 'GPP', kjit, gpp, kjpindex*nvm, indexveg)
       ENDIF
       IF ( control_in%ok_stomate ) THEN
             CALL histwrite_p(hist_id, 'NEE', kjit, co2_flux, kjpindex*nvm, indexveg)    
       ENDIF
    ENDIF ! almaoutput

    !! 2.13 Write additional output file with higher frequency
    IF ( hist2_id > 0 ) THEN
       IF ( .NOT. almaoutput ) THEN
          ! Write history file in IPSL-format
          CALL histwrite_p(hist2_id, 'sinang', kjit, 180/pi*acos(sinang), kjpindex, index)
          CALL histwrite_p(hist2_id, 'alb_vis_pft',kjit,albedo_pft(:,:,ivis),kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'alb_nir_pft',kjit,albedo_pft(:,:,inir),kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'tsol_rad', kjit, tsol_rad, kjpindex, index)
          CALL histwrite_p(hist2_id, 'qsurf', kjit, qsurf, kjpindex, index)
          CALL histwrite_p(hist2_id, 'emis', kjit, emis, kjpindex, index)
          CALL histwrite_p(hist2_id, 'beta', kjit, vbeta, kjpindex, index)
          CALL histwrite_p(hist2_id, 'z0', kjit, z0, kjpindex, index)
          CALL histwrite_p(hist2_id, 'roughheight', kjit, roughheight, kjpindex, index)
          CALL histwrite_p(hist2_id, 'vegetfrac', kjit, veget, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'maxvegetfrac', kjit, veget_max, kjpindex*nvm, indexveg)
          !CALL histwrite_p(hist2_id, 'nobiofrac', kjit, frac_nobio, kjpindex*nnobio, indexnobio)
          CALL histwrite_p(hist2_id, 'lai', kjit, lai, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'subli', kjit, vevapsno, kjpindex, index)
          IF ( control_in%do_floodplains ) THEN
             CALL histwrite_p(hist2_id, 'vevapflo', kjit, vevapflo, kjpindex, index)
             CALL histwrite_p(hist2_id, 'flood_frac', kjit, flood_frac, kjpindex, index)
          ENDIF
          CALL histwrite_p(hist2_id, 'vevapnu', kjit, vevapnu, kjpindex, index)
          CALL histwrite_p(hist2_id, 'transpir', kjit, transpir, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'transpot', kjit, transpot, kjpindex*nvm, indexveg)
          IF (control_in%ok_hydrol_arch) THEN
             CALL histwrite_p(hist2_id, 'transpir_supply', kjit, transpir_supply, kjpindex*nvm, indexveg)
          ENDIF
          !+++Temp+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
          !Add by YC
          !Here is trying to output the profile data to standard Sechiba2 output
          !++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
          !Separate the loop for each variable for the list in the output in an order of sequestration
          DO ilevel= 1,jnlvls
             !net radiation flux
             WRITE(var_name, "('RN_',i3.3)") ilevel
             CALL histwrite_p(hist2_id, var_name, kjit, flux_rn_grid(:,ilevel), kjpindex, index) 
          ENDDO
          DO ilevel= 1,jnlvls
             !sensibel heat flux
             WRITE(var_name, "('SH_',i3.3)") ilevel
             CALL histwrite_p(hist2_id, var_name, kjit, flux_h_grid(:,ilevel), kjpindex, index) 
          ENDDO  
          DO ilevel= 1,jnlvls
             !latent heat flux profile
             WRITE(var_name, "('LE_',i3.3)") ilevel
             CALL histwrite_p(hist2_id, var_name, kjit, flux_le_grid(:,ilevel), kjpindex, index) 
          ENDDO    
          DO ilevel= 1,jnlvls
             !wind profile
             WRITE(var_name, "('WS_',i3.3)") ilevel
             CALL histwrite_p(hist2_id, var_name, kjit, u_speed_grid(:,ilevel), kjpindex, index) 
          ENDDO        
          DO ilevel= 1,jnlvls   
             !air temperature profile
             WRITE(var_name, "('TA_',i3.3)") ilevel
             CALL histwrite_p(hist2_id, var_name, kjit, temp_atmos_pres_grid(:,ilevel), kjpindex, index) 
          ENDDO
          DO ilevel= 1,jnlvls 
             !leaf temperature profile
             WRITE(var_name, "('TL_',i3.3)") ilevel
             CALL histwrite_p(hist2_id, var_name, kjit, temp_leaf_pres_grid(:,ilevel), kjpindex, index)         
          ENDDO      
          DO ilevel= 1,jnlvls 
             !specific humidity profile
             WRITE(var_name, "('QA_',i3.3)") ilevel
             CALL histwrite_p(hist2_id, var_name, kjit, q_atmos_pres_grid(:,ilevel), kjpindex, index)         
          ENDDO
          !+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
          CALL histwrite_p(hist2_id, 'inter', kjit, vevapwet, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'vbeta1', kjit, vbeta1, kjpindex, index)
          CALL histwrite_p(hist2_id, 'vbeta2', kjit, vbeta2, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'vbeta3', kjit, vbeta3, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'vbeta3pot', kjit, vbeta3pot, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'vbeta4', kjit, vbeta4, kjpindex, index)    
          CALL histwrite_p(hist2_id, 'vbeta5', kjit, vbeta5, kjpindex, index)    
          CALL histwrite_p(hist2_id, 'drysoil_frac', kjit, drysoil_frac, kjpindex, index)
          CALL histwrite_p(hist2_id, 'rveget', kjit, rveget, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'rstruct', kjit, rstruct, kjpindex*nvm, indexveg)
          !+++++++++++++++++++++++++++++++++++++++++++++++++++
          ! Adde by YC 20140313  For Checking Energy Budget        
          CALL histwrite_p(hist2_id, 'fluxnet',  kjit, netrad_out,  kjpindex, index) 
          CALL histwrite_p(hist2_id, 'fluxsoil', kjit, soilflx_out, kjpindex, index) 
          !+++++++++++++++++++++++++++++++++++++++++++++++++++ 
          CALL histwrite_p(hist2_id, 'swe',      kjit, snow,        kjpindex, index) 
          
          IF ( .NOT. control_in%hydrol_cwrr ) THEN
             CALL histwrite_p(hist2_id, 'rsol', kjit, rsol, kjpindex, index)
          ENDIF
          CALL histwrite_p(hist2_id, 'snowage', kjit, snow_age, kjpindex, index)
          CALL histwrite_p(hist2_id, 'snownobio', kjit, snow_nobio, kjpindex*nnobio, indexnobio)
          CALL histwrite_p(hist2_id, 'snownobioage', kjit, snow_nobio_age, kjpindex*nnobio, indexnobio)
          !
          IF (  control_in%hydrol_cwrr ) THEN
             CALL histwrite_p(hist2_id, 'soilindex',  kjit, REAL(njsc, r_std), kjpindex, index)
             CALL histwrite_p(hist2_id, 'reinf_slope',  kjit, reinf_slope, kjpindex, index)
          ENDIF
          !
          IF ( control_in%ok_co2 ) THEN               
             CALL histwrite_p(hist2_id, 'gsmean', kjit, gsmean, kjpindex*nvm, indexveg)    
             CALL histwrite_p(hist2_id, 'gpp', kjit, gpp, kjpindex*nvm, indexveg)
             CALL histwrite_p(hist2_id, 'cimean', kjit, cimean, kjpindex*nvm, indexveg)    
          ENDIF
          IF ( control%ok_C13 ) THEN
             CALL histwrite_p(hist2_id, 'delta_c13_assim', kjit, delta_c13_assim, kjpindex*nvm, indexveg)
             CALL histwrite_p(hist2_id, 'leaf_ci_out', kjit, leaf_ci_out, kjpindex*nvm, indexveg)  
          ENDIF
          IF ( control_in%ok_stomate ) THEN
             CALL histwrite_p(hist2_id, 'nee', kjit, co2_flux, kjpindex*nvm, indexveg)    
          ENDIF
          ! For the snow fraction, we combine the snow fraction calculated for the
          ! vegetated and non-vegetated fractions to get a value for the whole pixel.
          frac_snow_pix(:)=zero
          DO ivm=1,nvm
             frac_snow_pix(:)=frac_snow_pix(:)+frac_snow_veg(:,ivm)*veget_max(:,ivm)
          ENDDO
          DO inno=1,nnobio
             frac_snow_pix(:)=frac_snow_pix(:)+frac_snow_nobio(:,inno)*frac_nobio(:,inno)
          ENDDO
          CALL histwrite_p(hist2_id, 'snowfrac', kjit, frac_snow_pix, kjpindex, index)
       ELSE
          ! Write history file in ALMA format
          CALL histwrite_p(hist2_id, 'sinang', kjit, 180/pi*acos(sinang), kjpindex, index)
          CALL histwrite_p(hist2_id, 'vegetfrac', kjit, veget, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'maxvegetfrac', kjit, veget_max, kjpindex*nvm, indexveg)
          CALL histwrite_p(hist2_id, 'nobiofrac', kjit, frac_nobio, kjpindex*nnobio, indexnobio)
          CALL histwrite_p(hist2_id, 'Qf', kjit, fusion, kjpindex, index)
          CALL histwrite_p(hist2_id, 'ESoil', kjit, vevapnu, kjpindex, index)
          IF ( control_in%do_floodplains ) THEN
             CALL histwrite_p(hist2_id, 'EWater', kjit, vevapflo, kjpindex, index)
             CALL histwrite_p(hist2_id, 'FloodFrac', kjit, flood_frac, kjpindex, index)
          ENDIF
          CALL histwrite_p(hist2_id, 'SWE', kjit, snow, kjpindex, index)
          histvar(:)=zero
          DO jv=1,nvm
             histvar(:) = histvar(:) + transpir(:,jv)
          ENDDO
          CALL histwrite_p(hist2_id, 'TVeg', kjit, histvar, kjpindex, index)
          histvar(:)=zero
          DO jv=1,nvm
             histvar(:) = histvar(:) + vevapwet(:,jv)
          ENDDO
          CALL histwrite_p(hist2_id, 'ECanop', kjit, histvar, kjpindex, index)
          CALL histwrite_p(hist2_id, 'ACond', kjit, tq_cdrag, kjpindex, index)
          IF ( control_in%ok_co2 ) THEN
             CALL histwrite_p(hist2_id, 'GPP', kjit, gpp, kjpindex*nvm, indexveg)
          ENDIF
          IF ( control_in%ok_stomate ) THEN
             CALL histwrite_p(hist2_id, 'NEE', kjit, co2_flux, kjpindex*nvm, indexveg)    
          ENDIF
       ENDIF ! almaoutput
    ENDIF ! hist2_id


!! 3. Write restart file for the next simulation from SECHIBA and other modules
    IF (ldrestart_write) THEN

       ! Only called during the last run
       IF (long_print) WRITE (numout,*) ' we have to write a restart file '
       !! 3.1 Call diffuco_main to write restart files
       CALL diffuco_main (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, &
            & index, indexveg, indexlai, u, v, &
            & zlev, z0, roughheight, temp_sol, temp_air, &
            & temp_growth, rau, tq_cdrag, qsurf, qair, &
            & q2m, t2m, pb, rsol, evap_bare_lim, &
            & evapot, evapot_corr, snow, flood_frac, flood_res, &
            & frac_nobio, snow_nobio, totfrac_nobio, swnet, swdown, &
            & sinang, ccanopy, humrel, vegstress, veget, &
            & veget_max, biomass, qsintveg, qsintmax, assim_param, &
            & vbeta, valpha, vbeta1, vbeta2, vbeta3, &
            & vbeta3pot, vbeta4, vbeta5, gsmean, rveget, &
            & rstruct, cimean, gpp, lalo, neighbours, &
            & resolution, ptnlev1, precip_rain, frac_age, rest_id, &
            & hist_id, hist2_id, vbeta2sum, vbeta3sum, Isotrop_Abs_Tot_p, &
            & Isotrop_Tran_Tot_p, lai_per_level, lai, vbeta23, leaf_ci, &
            & gs_distribution, gs_diffuco_output, gstot_component, gstot_frac, warnings, &
            & u_speed, profile_vbeta3, profile_rveget, delta_c13_assim, leaf_ci_out)

       !! 3.2 Call energy budget to write restart files
       !!     The third calling "enerbil_main" for writing restart files
        CALL enerbil_main (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, &
            & index, indexveg, zlev, lwdown, swnet, swdown, &
            & epot_air, temp_air, u, v, petAcoef, &
            & petBcoef, qair, peqAcoef, peqBcoef, pb, &
            & rau, vbeta, valpha, vbeta1, vbeta2, &
            & vbeta3, vbeta3pot, vbeta4, vbeta5, emis, &
            & soilflx, soilcap, tq_cdrag, humrel, fluxsens, &
            & fluxlat, vevapp, transpir, transpot, vevapnu, &
            & vevapwet, vevapsno, vevapflo, t2mdiag, temp_sol, &
            & tsol_rad, temp_sol_new, qsurf, evapot, evapot_corr, &
            & rest_id, hist_id, hist2_id, beta_diffuco, &
            & precip_rain, snow, sfcfrz, radsink, &
            & veget_max, osfcmelt, pgflux, gthick, gtemp, &
            & gpkappa, grndflux, albedo_undersnow, snowtemp, snowdz, &
            & radxs, pkappa_snow, precip_snow, snowliq, snowheat, & 
            & snowrho, snowgrain, snowdz_o, soilflxresid)

       !! 3.3 Call hydrology to write restart files
       IF ( .NOT. hydrol_cwrr ) THEN
          !! 3.3.1 Call water balance from Choisnel module (2 soil layers) to write restart file
          CALL hydrolc_main (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, &
               & index, indexveg, control_in, temp_sol_new, floodout, &
               & runoff, drainage, frac_nobio, totfrac_nobio, vevapwet, &
               & veget, veget_max, qsintmax, qsintveg, vevapnu, &
               & vevapsno, vevapflo, snow, snow_age, snow_nobio, &
               & snow_nobio_age, tot_melt, transpir, precip_rain, precip_snow, &
               & returnflow, reinfiltration, irrigation, humrel, vegstress, &
               & rsol, drysoil_frac, evapot, evapot_corr, flood_frac, &
               & flood_res, shumdiag, litterhumdiag, soilcap, rest_id, &
               & hist_id, hist2_id, temp_air, pb, u, v, &
               & swnet, albedo_undersnow, pgflux, snowrho, snowtemp, &
               & soiltemp, snowgrain, snowdz, snowheat, snowliq, &
               & sfcfrz, osfcmelt, grndflux, radsink, radxs, &
               & gtemp, soilflxresid)  

          evap_bare_lim(:) = -un
          k_litt(:) = huit
          shumdiag_perma(:,:)=shumdiag(:,:)
       ELSE
          !! 3.3.2 Call water balance from CWRR module (11 soil layers) to write restart file
          CALL hydrol_main (kjit, kjpindex, dtradia, ldrestart_read, ldrestart_write, &
               & index, indexveg, indexsoil, indexlayer, indexnbdl, &
               & control_in, temp_sol_new, floodout, runoff, drainage, &
               & frac_nobio, totfrac_nobio, vevapwet, veget, veget_max, &
               & njsc, qsintmax, qsintveg, vevapnu, vevapsno, &
               & vevapflo, snow, snow_age, snow_nobio, snow_nobio_age, &
               & tot_melt, transpir, precip_rain, precip_snow, returnflow, &
               & reinfiltration, irrigation, humrel, vegstress, drysoil_frac, &
               & evapot, evapot_corr, evap_bare_lim, flood_frac, flood_res, &
               & shumdiag, shumdiag_perma, k_litt, litterhumdiag, soilcap, &
               & soiltile, reinf_slope, rest_id, hist_id, hist2_id, &
               & swc, veg_soiltext, stempdiag, temp_air, pb, &
               & u, v, swnet, albedo_undersnow, pgflux, &
               & snowrho, snowtemp, soiltemp, snowgrain, snowdz, &
               & snowheat, snowliq, sfcfrz, grndflux, radsink, &
               & radxs, gtemp, soilflxresid)

          rsol(:) = -un
       ENDIF ! hydrol_cwrr
   
       !! 3.4 Call condveg to write surface variables to restart files
       CALL condveg_main (kjit, kjpindex, dtradia, ldrestart_read, &
            ldrestart_write, index, lalo, neighbours, resolution, contfrac, &
            veget, veget_max, frac_nobio, totfrac_nobio, zlev, &
            snow, snow_age, snow_nobio, snow_nobio_age, drysoil_frac, &
            height,  emis, albedo, z0, roughheight, &
            rest_id, hist_id, hist2_id, lai,  sinang, &
            circ_class_biomass, circ_class_n, lai_split, z0_veg, Isotrop_Abs_Tot_p, &
            Isotrop_Tran_Tot_p, laieff_fit, albedo_pft, frac_snow_nobio, frac_snow_veg, &
            albedo_undersnow, snowdz, snowrho, snowliq, precip_snow, &
            osfcmelt, temp_air, u, v, &
            Collim_Abs_Tot, Collim_Alb_Tot, laieff_collim, laieff_isotrop)
 
       !! 3.5 Call soil thermodynamic to write restart files
       CALL thermosoil_main (kjit, kjpindex, dtradia, & 
            & ldrestart_read, ldrestart_write, index, indexgrnd,indexnbdl, & 
            & control_in,temp_sol_new, snow, soilcap, soilflx, shumdiag_perma, & 
            & stempdiag, ptnlev1, rest_id, hist_id, hist2_id, & 
            & soiltemp,pb,grndflux,snowrho,snowdz,snowtemp,gthick,gtemp,gpkappa,& 
            & pkappa_snow,snowdz_o) 

       !! 3.6 Add river routing to restart files  
       IF ( river_routing .AND. nbp_glo .GT. 1) THEN
          !! 3.6.1 Call river routing to write restart files 
          CALL routing_main (kjit, kjpindex, dtradia, control_in, ldrestart_read, ldrestart_write, index, &
               & lalo, neighbours, resolution, contfrac, totfrac_nobio, veget_max, floodout, runoff, &
               & drainage, transpot, precip_rain, humrel, k_litt, flood_frac, flood_res, &
               & stempdiag, reinf_slope, returnflow, reinfiltration, irrigation, riverflow, coastalflow, &
               & rest_id, hist_id, hist2_id)
       ELSE
          !! 3.6.2 No routing, set variables to zero
          riverflow(:) = zero
          coastalflow(:) = zero
          reinfiltration(:) = zero
          returnflow(:) = zero
          irrigation(:) = zero
          flood_frac(:) = zero
          flood_res(:) = zero
       ENDIF ! river_routing

       !! 3.7 Call slowproc_main to add 'daily' and annual variables to restart file
       CALL slowproc_main (kjit, kjpij, kjpindex, dtradia, date0, &
            ldrestart_read, ldrestart_write, control_in%ok_co2, control_in%ok_stomate, index, &
            indexveg, lalo, neighbours, resolution, contfrac, &
            soiltile, reinf_slope, t2mdiag, t2mdiag, temp_sol, &
            stempdiag, vegstress, shumdiag, litterhumdiag, precip_rain, &
            precip_snow, gpp, deadleaf_cover, assim_param, lai, &
            frac_age, height, veget, frac_nobio, njsc, &
            veget_max, totfrac_nobio, qsintmax, rest_id, hist_id, &
            hist2_id, rest_id_stom, hist_id_stom, hist_id_stom_IPCC, co2_flux, &
            fco2_lu, sinang, circ_class_biomass, circ_class_n, &
            transpir, stressed, unstressed, transpir_supply, &
            biomass, ind, lai_per_level, laieff_fit, &
            vir_transpir_supply, temp_growth, Isotrop_Tran_Tot_p, &
            sum_veget_diff, snow, jnlevels_loc, h_array_out, &
            z_array_out, max_height_store, t2m_month_out)

       ! Compute global CO2 flux 
       !+++CHECK+++
       ! If co2flux contains soil respiration the loop should include the bare soil
       ! because after a land cover change PFT1 can contain litter and soil carbon
       netco2flux(:) = zero
       DO jv = 2,nvm
          netco2flux(:) = netco2flux(:) + co2_flux(:,jv)*veget_max(:,jv)
       ENDDO
       !+++++++++++

       !! 3.8 Add some remaining variables to the restart file
       var_name= 'shumdiag'  
       CALL restput_p(rest_id, var_name, nbp_glo,   nbdl, 1, kjit,  shumdiag, 'scatter',  nbp_glo, index_g)
       var_name= 'runoff'  
       CALL restput_p(rest_id, var_name, nbp_glo,   1, 1, kjit,  runoff, 'scatter',  nbp_glo, index_g)
       var_name= 'drainage'  
       CALL restput_p(rest_id, var_name, nbp_glo,   1, 1, kjit,  drainage, 'scatter',  nbp_glo, index_g)
    
    END IF ! ldrestart_write 
  
   

    IF (long_print) WRITE (numout,*) ' sechiba_main done '
    
  END SUBROUTINE sechiba_main

  
!! ==============================================================================================================================\n
!! SUBROUTINE 	: sechiba_init
!!
!>\BRIEF        Dynamic allocation of the variables, the dimensions of the 
!! variables are determined by user-specified settings. If ::ldrestart_read is
!! true, initial varaible values are read from the restart file.
!! 
!! DESCRIPTION  : The domain size (:: kjpindex) is used to allocate the correct
!! dimensions to all variables in sechiba. Depending on the variable, its 
!! dimensions are also determined by the number of PFT's (::nvm), number of 
!! soil types (::nstm), number of non-vegetative surface types (::nnobio),
!! number of soil levels (::ngrnd), number of soil layers in the hydrological 
!! model (i.e. cwrr) (::nslm). Values for these variables are set in
!! constantes_soil.f90 and constantes_veg.f90.\n
!!
!! Memory is allocated for all Sechiba variables and new indexing tables
!! are build making use of both (::kjpij) and (::kjpindex). New indexing tables 
!! are needed because a single pixel can contain several PFTs, soil types, etc.
!! The new indexing tables have separate indices for the different
!! PFTs, soil types, etc.\n
!!
!! The routine uses the flag ::ldrestart_read to retrieve initial variable 
!! settings from the restart file.\n
!!  
!! ??Flags are getting swapped by it is not entirly clear why?? 
!!
!! RECENT CHANGE(S): None
!! 
!! MAIN OUTPUT VARIABLE(S): Strictly speaking the subroutine has no output 
!! variables. However, the routine allocates memory and builds new indexing 
!! variables for later use.
!!
!! REFERENCE(S)	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================ 

  SUBROUTINE sechiba_init (kjit, ldrestart_read, kjpij, kjpindex, index, rest_id, control_in, lalo)

!! 0.1 Input variables
 
    INTEGER(i_std), INTENT (in)                         :: kjit               !! Time step number (unitless)
    INTEGER(i_std), INTENT (in)                         :: kjpij              !! Total size of the un-compressed grid (unitless)
    INTEGER(i_std), INTENT (in)                         :: kjpindex           !! Domain size - terrestrial pixels only (unitless)
    INTEGER(i_std), INTENT (in)                         :: rest_id            !! _Restart_ file identifier (unitless)
    TYPE(control_type), INTENT(in)                      :: control_in         !! Flags that (de)activate parts of the model
    LOGICAL,INTENT (in)                                 :: ldrestart_read     !! Logical for restart file to read (true/false)
    INTEGER(i_std),DIMENSION (kjpindex), INTENT (in)    :: index              !! Indeces of the points on the map (unitless)
    REAL(r_std),DIMENSION (kjpindex,2), INTENT (in)     :: lalo               !! Geographical coordinates (latitude,longitude) 
                                                                              !! for pixels (degrees)
!! 0.2 Output variables

!! 0.3 Modified variables

!! 0.4 Local variables

    INTEGER(i_std)                                      :: ier                !! Check errors in memory allocation (unitless)
    INTEGER(i_std)                                      :: ji, jv, ilevel     !! Indeces (unitless)
    INTEGER(i_std)                                      :: icir               !! Indeces (unitless)
!_ ==============================================================================================================================

!! 1. Initialize variables 
    
    ! Dynamic allocation with user-specified dimensions on first call
    IF (l_first_sechiba) THEN 
       l_first_sechiba=.FALSE.
    ELSE 
       WRITE (numout,*) ' l_first_sechiba false . we stop '
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    ENDIF


    !! 1.1 Initialize 3D vegetation indexation table
    ALLOCATE (indexveg(kjpindex*nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexveg allocation. We stop. We need kjpindex words = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (indexlai(kjpindex*(nlai+1)),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexlai allocation. We stop. We need kjpindex words = ',kjpindex*(nlai+1)
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (indexsoil(kjpindex*nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexsoil allocation. We stop. We need kjpindex words = ',kjpindex*nstm
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (indexnobio(kjpindex*nnobio),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexnobio allocation. We stop. We need kjpindex words = ',kjpindex*nnobio
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (indexgrnd(kjpindex*ngrnd),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexgrnd allocation. We stop. We need kjpindex words = ',kjpindex*ngrnd
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (indexsnow(kjpindex*nsnow),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in indexsnow allocation. We stop. We need kjpindex words = ',kjpindex*nsnow 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    END IF 

    ALLOCATE (indexlayer(kjpindex*nslm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexlayer allocation. We stop. We need kjpindex words = ',kjpindex*nslm
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (indexnbdl(kjpindex*nbdl),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in indexgrnd allocation. We stop. We need kjpindex words = ',kjpindex*nbdl 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    END IF 

    ALLOCATE (indexalb(kjpindex*2),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexalb allocation. We stop. We need kjpindex words = ',kjpindex*2
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (indexcan(kjpindex*nlevels_tot),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in indexcan allocation. We stop. We need kjpindex*nlevels_tot words = ',&
            kjpindex*nlevels_tot
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    !! 1.2  Initialize 1D array allocation with restartable value
    IF (long_print) WRITE (numout,*) 'Allocation of 1D variables. We need for each kjpindex words = ',kjpindex
    ALLOCATE (flood_res(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in flood_res allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    flood_res(:) = undef_sechiba

    IF (long_print) WRITE (numout,*) 'Allocation of 1D variables. We need for each kjpindex words = ',kjpindex
    ALLOCATE (flood_frac(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in flood_frac allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    flood_frac(:) = undef_sechiba

    IF (long_print) WRITE (numout,*) 'Allocation of 1D variables. We need for each kjpindex words = ',kjpindex
    ALLOCATE (snow(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in snow allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    snow(:) = undef_sechiba

    ALLOCATE (snow_age(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in snow_age allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    snow_age(:) = undef_sechiba

    ALLOCATE (drysoil_frac(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in drysoil_frac allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    drysoil_frac(:) = zero

    ALLOCATE (rsol(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in rsol allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (evap_bare_lim(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in evap_bare_lim allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (evapot(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in evapot allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    evapot(:) = undef_sechiba

    ALLOCATE (evapot_corr(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in evapot_corr allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (humrel(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in humrel allocation. We stop. we need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    humrel(:,:) = undef_sechiba

    ALLOCATE (vegstress(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in vegstress allocation. We stop. we need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    vegstress(:,:) = undef_sechiba

  !  ALLOCATE (laipix(kjpindex),stat=ier)
  !  IF (ier.NE.0) THEN
  !     WRITE (numout,*) ' error in laipix allocation. We stop. we need kjpindex words = ',kjpindex
  !    CALL ipslerr_p (3,'sechiba','sechiba_init','','')
  !  END IF
  !  laipix(:) = undef_sechiba

  !  ALLOCATE (laieff_pix(kjpindex),stat=ier)
  !  IF (ier.NE.0) THEN
  !     WRITE (numout,*) ' error in laieff_pix allocation. We stop. We need kjpindex x nvm words = ',&
  !          & kjpindex
  !    CALL ipslerr_p (3,'sechiba','sechiba_init','','')
  !  END IF
  !  laieff_pix(:) = zero

    ALLOCATE (njsc(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in njsc allocation. We stop. we need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    njsc(:)= undef_int

    ALLOCATE (soiltile(kjpindex,nstm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in soiltile allocation. We stop. we need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    soiltile(:,:)=undef_sechiba

    ALLOCATE (reinf_slope(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in reinf_slope allocation. We stop. we need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    reinf_slope(:)=undef_sechiba

    ALLOCATE (vbeta1(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in vbeta1 allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (vbeta4(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in vbeta4 allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (vbeta5(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in vbeta5 allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (soilcap(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in soilcap allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (soilflx(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in soilflx allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (temp_sol(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in temp_sol allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    temp_sol(:) = undef_sechiba

    ALLOCATE (qsurf(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in qsurf allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    qsurf(:) = undef_sechiba

    !! 1.3 Initialize 2D array allocation with restartable value
    ALLOCATE (qsintveg(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in qsintveg allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm,' = ',kjpindex*nvm 
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    qsintveg(:,:) = undef_sechiba

    ALLOCATE (vbeta2(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in vbeta2 allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (vbeta3(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in vbeta3 allocation. We stop.We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (vbeta3pot(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in vbeta3pot allocation. We stop.We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (gsmean(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in gsmean allocation. We stop.We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (cimean(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in cimean allocation. We stop.We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (gpp(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in gpp allocation. We stop.We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    gpp(:,:) = undef_sechiba

    ALLOCATE (temp_growth(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in temp_growth allocation. We stop.We need kjpindex words = ',&
            & kjpindex,' = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    temp_growth(:) = undef_sechiba

    ALLOCATE (veget(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in veget allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    veget(:,:)=undef_sechiba

    ALLOCATE (veget_max(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in veget_max allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    veget_max(:,:)=undef_sechiba

    ALLOCATE (lai(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in lai allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    lai(:,:)=undef_sechiba


    ! Here I would rather declare a structure to pass around.  This way it's 
    ! easier to change the function we fit this data to.
    ALLOCATE (laieff_fit(kjpindex,nvm,nlevels_tot),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in laieff_fit allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',nlevels_tot*kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    CALL laieff_type_init(kjpindex, nlevels_tot, laieff_fit)
    
    ALLOCATE (frac_age(kjpindex,nvm,nleafages),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in frac_age allocation. We stop. We need kjpindex x nvm words = ',&
             & kjpindex,' x ' ,nvm, 'x',nleafages,' = ',kjpindex*nvm*nleafages
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    frac_age(:,:,:)=undef_sechiba

    ALLOCATE (height(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in height allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    height(:,:)=undef_sechiba

    ALLOCATE (frac_nobio(kjpindex,nnobio),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in frac_nobio allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    frac_nobio(:,:) = undef_sechiba

    ALLOCATE (albedo(kjpindex,2),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in albedo allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (albedo_pft(kjpindex,nvm,n_spectralbands),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in albedo_pft allocation. We stop. ' // &
            'We need kjpindex*nvm*n_spectralbands = ',&
            kjpindex*nvm*n_spectralbands
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (snow_nobio(kjpindex,nnobio),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in snow_nobio allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    snow_nobio(:,:) = undef_sechiba

    ALLOCATE (snow_nobio_age(kjpindex,nnobio),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in snow_nobio_age allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    snow_nobio_age(:,:) = undef_sechiba

    ALLOCATE (assim_param(kjpindex,nvm,npco2),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in assim_param allocation. We stop. We need kjpindex x nvm x npco2 words = ',&
            & kjpindex,' x ' ,nvm,' x ',npco2, ' = ',kjpindex*nvm*npco2
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (delta_c13_assim(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in delta_c13_assim allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
       STOP 'sechiba_init'
    END IF
    delta_c13_assim(:,:)=undef_sechiba

    ALLOCATE (leaf_ci_out(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in leaf_ci_out allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
       STOP 'sechiba_init'
    END IF
    leaf_ci_out(:,:)=undef_sechiba


    !! 1.4 Initialize 1D array allocation 
    ALLOCATE (vevapflo(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in vevapflo allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    vevapflo(:)=zero

    ALLOCATE (vevapsno(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in vevapsno allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (vevapnu(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in vevapnu allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (t2mdiag(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in t2mdiag allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (totfrac_nobio(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in totfrac_nobio allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (floodout(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in floodout allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (runoff(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in runoff allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (drainage(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in drainage allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (returnflow(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in returnflow allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    returnflow(:) = zero

    ALLOCATE (reinfiltration(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in reinfiltration allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    reinfiltration(:) = zero

    ALLOCATE (irrigation(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in irrigation allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    irrigation(:) = zero

    ALLOCATE (z0(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in z0 allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (z0_veg(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in z0_veg allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (roughheight(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in roughheight allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    ! Initialize it here, otherwise it gets used somewhere before being
    ! set.  I wasn't able to find where, though.
    roughheight(:)=zero

    ALLOCATE (emis(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in emis allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (tot_melt(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in tot_melt allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (valpha(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in valpha allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (vbeta(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in vbeta allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (fusion(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in fusion allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (rau(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in rau allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (deadleaf_cover(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in deadleaf_cover allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (stempdiag(kjpindex, nbdl),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in stempdiag allocation. We stop. We need kjpindex*nbdl words = ',&
            & kjpindex*nbdl
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (co2_flux(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in co2_flux allocation. We stop. We need kjpindex*nvm words = ' ,kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    co2_flux(:,:)=zero

    ALLOCATE (shumdiag(kjpindex,nbdl),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in shumdiag allocation. We stop. We need kjpindex*nbdl words = ',&
            & kjpindex*nbdl
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    
    ALLOCATE (shumdiag_perma(kjpindex,nbdl),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in shumdiag_perma allocation. We stop. We need kjpindex*nbdl words = ',&
            & kjpindex*nbdl
       STOP 'sechiba_init'
    END IF

    ALLOCATE (litterhumdiag(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in litterhumdiag allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (ptnlev1(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in ptnlev1 allocation. We stop. We need kjpindex words = ', kjpindex
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (k_litt(kjpindex),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in k_litt allocation. We stop. We need kjpindex words = ',kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    !! 1.5 Initialize 2D array allocation
    ALLOCATE (vevapwet(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in vevapwet allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    vevapwet(:,:)=undef_sechiba

    ALLOCATE (transpir(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in transpir allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (stressed(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in stressed allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    stressed(:,:) = zero

     ALLOCATE (unstressed(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in unstressed allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    unstressed(:,:) = zero

!!$    ALLOCATE (vir_transpir_mod(kjpindex,nvm),stat=ier)
!!$    IF (ier.NE.0) THEN
!!$       WRITE (numout,*) ' error in vir_transpir_mod allocation. We stop. We need kjpindex x nvm words = ',&
!!$            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
!!$      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
!!$    END IF
!!$    vir_transpir_mod(:,:) = zero 

    ALLOCATE (transpir_supply(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in transpir_supply allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    transpir_supply(:,:) = zero

    ALLOCATE (vir_transpir_supply(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in transpir_supply allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    vir_transpir_supply(:,:) = zero

    ALLOCATE (transpot(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in transpot allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (qsintmax(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in qsintmax allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm 
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (rveget(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in rveget allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (rstruct(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in rstruct allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' = ',kjpindex*nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (warnings(kjpindex,nvm,nwarns),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in warnings allocation. We stop. We need kjpindex x nvm x nwarns words = ',&
            & kjpindex,' x ' ,nvm, ' x ',nwarns,' = ',kjpindex*nvm*nwarns
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    ! We aren't restarting the warnings at the moment.
    warnings(:,:,:)=zero


    !! 1.? Initialize 3D array allocation
    ALLOCATE (Isotrop_Abs_Tot_p(kjpindex,nvm,nlevels_tot),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in Isotrop_Abs_Tot_p allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' x ' , nlevels, ' = ',kjpindex*nvm*nlevels_tot
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    Isotrop_Abs_Tot_p(:,:,:)= zero

    ALLOCATE (Isotrop_Tran_Tot_p(kjpindex,nvm,nlevels_tot),stat=ier)
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in Isotrop_Tran_Tot_p allocation. We stop. We need kjpindex x nvm words = ',&
            & kjpindex,' x ' ,nvm, ' x ' , nlevels, ' = ',kjpindex*nvm*nlevels_tot
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF
    Isotrop_Tran_Tot_p(:,:,:)= zero

    ALLOCATE (osfcmelt(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in osfcmelt allocation. We stop. We need kjpindex  words = ',& 
            & kjpindex, ' = ',kjpindex 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    END IF
    osfcmelt(:) = .FALSE. 

    ALLOCATE(max_height_store(kjpindex,nvm),stat=ier)
    IF (ier.NE.0) THEN
       WRITE(numout,*) ' Memory allocation error for max_height_store. ' // &
            'We stop. We need npts*nvm = ',&
            kjpindex*nvm
       STOP 'stomate_init'
    ENDIF

    ALLOCATE (albedo_undersnow(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in albedo_undersnow allocation. We stop. We need kjpindex words = ',kjpindex 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    END IF

    ALLOCATE (pgflux(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in pgflux allocation. We stop. We need kjpindex words = ',kjpindex 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    END IF
    pgflux(:)= 0.0 

    ALLOCATE (grndflux(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in grndflux allocation. We stop. We need kjpindex words = ',& 
            & kjpindex, ' = ',kjpindex 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    grndflux(:) = 0.0 

    ALLOCATE (radxs(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in radxs allocation. We stop. We need kjpindex words = ',& 
            & kjpindex, ' = ',kjpindex 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    radxs(:) = 0.0 

    ALLOCATE (pkappa_snow(kjpindex,nsnow),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in pkappa_snow allocation. We stop. We need kjpindex x nsnow words = ',& 
            & kjpindex,' x ' ,nsnow, ' = ',kjpindex*nsnow 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    pkappa_snow(:,:) = 0.0 

    ALLOCATE (gthick(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in gthick allocation. We stop. We need kjpindex words = ',kjpindex 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    gthick(:) = undef_sechiba 

    ALLOCATE (gtemp(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in gtemp allocation. We stop. We need kjpindex words = ',kjpindex 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    gtemp(:) = 280.0 

    ALLOCATE (soilflxresid(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in soilflxresid allocation. We stop. We need kjpindex words = ',kjpindex 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    soilflxresid(:) = 0.0 

    ALLOCATE (gpkappa(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in gpkappa allocation. We stop. We need kjpindex words = ',kjpindex 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    gpkappa(:) =undef_sechiba 

    ALLOCATE (sfcfrz(kjpindex),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in sfcfrz allocation. We stop. We need kjpindex words = ',kjpindex 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF

    ALLOCATE (radsink(kjpindex,nsnow),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in radsink allocation. We stop. We need kjpindex x nsnow words = ',& 
            & kjpindex,' x ' ,nsnow, ' = ',kjpindex*nsnow 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    radsink(:,:)=0 

    ALLOCATE (snowrho(kjpindex,nsnow),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in snowrho allocation. We stop. We need kjpindex x nsnow words = ',& 
            & kjpindex,' x ' ,nsnow, ' = ',kjpindex*nsnow 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    snowrho(:,:) = xrhosmin 

    ALLOCATE (snowheat(kjpindex,nsnow),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in snowheat allocation. We stop. We need kjpindex x nsnow  words = ',& 
            & kjpindex,' x ' ,nsnow, ' = ',kjpindex*nsnow 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    snowheat(:,:) =0.0 

    ALLOCATE (snowliq(kjpindex,nsnow),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in snowliq allocation. We stop. We need kjpindex x nsnow words = ',& 
            & kjpindex,' x ' ,nsnow, ' = ',kjpindex*nsnow 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    snowliq(:,:) = 0.0 

    ALLOCATE (snowgrain(kjpindex,nsnow),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in snowgrain allocation. We stop. We need kjpindex x nsnow words = ',& 
            & kjpindex,' x ' ,nsnow, ' = ',kjpindex*nsnow 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    snowgrain(:,:) = 0.0 

    ALLOCATE (snowtemp(kjpindex,nsnow),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in snowtemp allocation. We stop. We need kjpindex x nsnow words = ',& 
            & kjpindex,' x ' ,nsnow, ' = ',kjpindex*nsnow 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    snowtemp(:,:) = tp_00 

    ALLOCATE (soiltemp(kjpindex,ngrnd),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in soiltemp allocation. We stop. We need kjpindex x ngrnd words = ',& 
            & kjpindex,' x ' ,ngrnd, ' = ',kjpindex*ngrnd 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    soiltemp(:,:) = tp_00 

    ALLOCATE (snowdz(kjpindex,nsnow),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in snowdepth allocation. We stop. We need kjpindex x nsnow words = ',& 
            & kjpindex,' x ' ,nsnow, ' = ',kjpindex*nsnow 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    snowdz(:,:)=0.0 

    ALLOCATE (snowdz_o(kjpindex,nsnow),stat=ier) 
    IF (ier.NE.0) THEN 
       WRITE (numout,*) ' error in snowdepth allocation. We stop. We need kjpindex x nsnow words = ',& 
            & kjpindex,' x ' ,nsnow, ' = ',kjpindex*nsnow 
       CALL ipslerr_p (3,'sechiba','sechiba_init','','') 
    ENDIF
    snowdz_o(:,:)=0.0 

    !! 1.6 Initialize indexing table for the vegetation fields. 
    ! In SECHIBA we work on reduced grids but to store in the full 3D filed vegetation variable 
    ! we need another index table : indexveg, indexsoil, indexnobio and indexgrnd
    DO ji = 1, kjpindex
       !
       DO jv = 1, nlai+1
          indexlai((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij + offset_omp - offset_mpi
       ENDDO
       !
       DO jv = 1, nvm
          indexveg((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij + offset_omp - offset_mpi
       ENDDO
       !      
       DO jv = 1, nstm
          indexsoil((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij + offset_omp - offset_mpi
       ENDDO
       !      
       DO jv = 1, nnobio
          indexnobio((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij + offset_omp - offset_mpi
       ENDDO
       !
       DO jv = 1, ngrnd
          indexgrnd((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij + offset_omp - offset_mpi
       ENDDO
       !
       DO jv = 1, nsnow 
          indexsnow((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij 
       ENDDO 
       !
       DO jv = 1, nbdl 
          indexnbdl((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij 
       ENDDO
       !
       DO jv = 1, nslm
          indexlayer((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij + offset_omp - offset_mpi
       ENDDO
       !
       DO jv = 1, 2
          indexalb((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij + offset_omp - offset_mpi
       ENDDO
       !
       DO jv = 1, nlevels_tot
          indexcan((jv-1)*kjpindex + ji) = INDEX(ji) + (jv-1)*kjpij + offset_omp - offset_mpi
       ENDDO
       !
    ENDDO

   !! 1.7 Initialize heights of the levels in the canopy
    ALLOCATE(lai_split(1:nlevels),stat=ier)
    IF (ier .NE. 0) THEN
       WRITE (numout,*) ' error in lai_split allocation. We stop. We need nlevels words = ',nlevels
       CALL ipslerr_p (3,'sechiba','config_sechiba_parameters','','')
    ENDIF
    IF(nlevels .GT. 1)THEN
       !Config Key   = LAI_SPLIT
       !Config Desc  = 
       !Config If    = OK_SECHIBA
       !Config Def   = 0.0
       !Config Help  = 
       !Config Units = [-]
       lai_split(:)=zero
       CALL getin_p('LAI_SPLIT',lai_split(:))

       WRITE(numout,*) 'WARNING: Using a temporary value LAI_SPLIT'
       WRITE(numout,*) '         There is no reason to use this until you are a developer!'
    ELSE
       lai_split(1)=zero
    ENDIF

    DO ilevel=1,nlevels
       WRITE(numout,*) 'Albedo canopy level ',ilevel,' has a lower boundary at ',lai_split(ilevel),' meters.'
    ENDDO

    ! now we create and initialize some plant variables that need to be passed around sechiba and stomate
    ALLOCATE(circ_class_n(kjpindex,nvm,ncirc),stat=ier)
    IF (ier .NE. 0) THEN
       WRITE(numout,*) 'Memory allocation error for circ_class_n. We stop. We need kjpindex*nvm*ncirc words', & 
       &      kjpindex,nvm,ncirc
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    ENDIF
    circ_class_n(:,:,:) = val_exp
    
    ALLOCATE(circ_class_biomass(kjpindex,nvm,ncirc,nparts,nelements),stat=ier)
    IF (ier .NE. 0) THEN
       WRITE(numout,*) 'Memory allocation error for circ_class_biomass.'
       WRITE(numout,*) 'We stop. We need kjpindex*nvm*nparts*ncirc*nelmements words', & 
       &      kjpindex,nvm,ncirc,nparts,nelements
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    ENDIF
    circ_class_biomass(:,:,:,:,:) = val_exp


    ALLOCATE(biomass(kjpindex,nvm,nparts,nelements),stat=ier)
    IF (ier .NE. 0) THEN
       WRITE(numout,*) 'Memory allocation error for biomass.'
       WRITE(numout,*) 'We stop. We need kjpindex*nvm*nparts*nelmements words', & 
       &      kjpindex,nvm,nparts,nelements
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    ENDIF
    biomass(:,:,:,:) = val_exp

    ALLOCATE(ind(kjpindex,nvm),stat=ier)
    IF (ier .NE. 0) THEN
       WRITE(numout,*) 'Memory allocation error for biomass.'
       WRITE(numout,*) 'We stop. We need kjpindex*nvm words', & 
       &      kjpindex,nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    ENDIF
    ind(:,:) = val_exp


    ALLOCATE(lai_per_level(kjpindex,nvm,nlevels_tot),stat=ier)
    IF (ier .NE. 0) THEN
       WRITE(numout,*) 'Memory allocation error for lai_per_level. We stop. ' //&
            'We need kjpindex*nvm*nlevels_tot words', & 
       &      kjpindex,nvm,nlevels_tot
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    ENDIF
    lai_per_level(:,:,:)=val_exp

    ALLOCATE(frac_snow_veg(kjpindex,nvm),stat=ier)
    IF (ier .NE. 0) THEN
       WRITE(numout,*) 'Memory allocation error for frac_snow_veg. We stop. ' //&
            'We need kjpindex*nvm words', & 
       &      kjpindex,nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    ENDIF
    frac_snow_veg(:,:)=val_exp

    ALLOCATE(frac_snow_nobio(kjpindex,nvm),stat=ier)
    IF (ier .NE. 0) THEN
       WRITE(numout,*) 'Memory allocation error for frac_snow_nobio. We stop. ' //&
            'We need kjpindex*nvm words', & 
       &      kjpindex,nvm
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    ENDIF
    frac_snow_nobio(:,:)=val_exp

    ALLOCATE(frac_snow_pix(kjpindex),stat=ier)
    IF (ier .NE. 0) THEN
       WRITE(numout,*) 'Memory allocation error for frac_snow_pix. We stop. ' //&
            'We need kjpindex words', & 
            &      kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    ENDIF
    frac_snow_pix(:)=val_exp

    ! This is really just a debug vector.
    ALLOCATE(sum_veget_diff(kjpindex),stat=ier)
    IF (ier .NE. 0) THEN
       WRITE(numout,*) 'Memory allocation error for sum_veget_diff. We stop. ' //&
            'We need kjpindex words', & 
            &      kjpindex
      CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    ENDIF
    sum_veget_diff(:)=val_exp


    !multilayer allocations

    ALLOCATE (flux_rn_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in flux_rn_grid allocation. We stop. We need kjpindex words = ',jnlvls
        STOP 'sechiba_init'
    END IF

    ALLOCATE (flux_h_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in flux_h_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (flux_le_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in flux_le_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (u_speed_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in u_speed_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (temp_atmos_pres_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in temp_atmos_pres_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (q_atmos_pres_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in q_atmos_pres_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (temp_leaf_pres_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in temp_leaf_pres_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (t_a_next_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in t_a_next_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (q_a_next_grid(kjpindex,jnlvls),stat=ier)
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in q_a_next_grid allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (swc(kjpindex,nslm,nstm))
    IF (ier.NE.0) THEN
        WRITE (numout,*) ' error in swc allocation. We stop. We need kjpindex words = ',jnlvls
       CALL ipslerr_p (3,'sechiba','sechiba_init','','')
    END IF

    ALLOCATE (h_array_out(kjpindex,nvm,ncirc,nlevels_tot),STAT=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in h_array_out allocation. We stop. We need kjpindex*nvm words = ',kjpindex*nvm*ncirc*nlevels_tot
       STOP
    END IF

    ALLOCATE (z_array_out(kjpindex,nvm,ncirc,nlevels_tot),STAT=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in z_array_out allocation. We stop. We need kjpindex*nvm words = ',kjpindex*nvm*ncirc*nlevels_tot
       STOP
    END IF

    ALLOCATE (t2m_month_out(kjpindex),STAT=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in t2m_month_out allocation. We stop. We need kjpindex words = ',kjpindex
       STOP
    END IF
    ALLOCATE (profile_vbeta3(kjpindex,nvm,nlevels_tot),STAT=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in profile_vbeta3 allocation. We stop. We need kjpindex*nvm words = ',kjpindex*nvm*nlevels_tot
       STOP
    END IF

    ALLOCATE (profile_rveget(kjpindex,nvm,nlevels_tot),STAT=ier) 
    IF (ier.NE.0) THEN
       WRITE (numout,*) ' error in profile_rveget allocation. We stop. We need kjpindex*nvm words = ',kjpindex*nvm*nlevels_tot
       STOP
    END IF



!! 2. Read restart files to set initial variable values
    
    IF (ldrestart_read) THEN

       IF (long_print) WRITE (numout,*) ' we have to read a restart file for SECHIBA variables'
       ! Open restart file and read data
       ! ?? Where is the code to open the restart file?? !!?? I guess, the IF just writes out the comment.
       !!?? The single routines are called in sechiba_main. It would be more logical to put it in sechiba_main
       ! Read the default value that will be put into variable which are not in the restart file
       CALL ioget_expval(val_exp)

    ENDIF

!! 3. Swapping flags

    !??What is going with the flags (also next section??). Why is this done?? Where are the values specified??
    river_routing = control_in%river_routing
    hydrol_cwrr = control_in%hydrol_cwrr
    
!! 4. Run control: store flags in a common variable

    control%river_routing = control_in%river_routing
    control%hydrol_cwrr = control_in%hydrol_cwrr
    control%ok_co2 = control_in%ok_co2
    control%ok_sechiba = control_in%ok_sechiba
    control%ok_stomate = control_in%ok_stomate
    control%ok_dgvm = control_in%ok_dgvm
    control%do_land_use = control_in%do_land_use
    control%ok_pheno = control_in%ok_pheno
    control%stomate_watchout = control_in%stomate_watchout
    control%ok_functional_allocation = control_in%ok_functional_allocation
    control%ok_constant_mortality = control_in%ok_constant_mortality
    control%ok_windfall = control_in%ok_windfall
    control%ok_herbivory = control_in%ok_herbivory
    control%ok_constant_mortality = control_in%ok_constant_mortality
    control%ok_cexchange = control_in%ok_cexchange
    control%ok_inca = control_in%ok_inca                    
    control%ok_leafage = control_in%ok_leafage         
    control%ok_radcanopy = control_in%ok_radcanopy
    control%ok_multilayer = control_in%ok_multilayer
    control%ok_pulse_NOx = control_in%ok_pulse_NOx       
    control%ok_bbgfertil_NOx = control_in%ok_bbgfertil_NOx     
    control%ok_cropsfertil_NOx = control_in%ok_cropsfertil_NOx
    control%forest_management = control_in%forest_management 
    control%do_new_snow_albedo = control_in%do_new_snow_albedo
    control%ok_hydrol_arch = control_in%ok_hydrol_arch
    control%ok_gs_feedback = control_in%ok_gs_feedback
    control%ok_dofoco = control_in%ok_dofoco
    control%ok_windfall = control_in%ok_windfall
    control%ok_c13 = control_in%ok_c13
    control%ok_nenerbil = control_in%ok_nenerbil
    control%ok_new_enerbil_nextstep = control_in%ok_new_enerbil_nextstep

    ! +++CHECK+++
    ! The ok_land_use_change flag replace the old logical variable :lcchange
    ! the use has not changed it is just more consistent to define all flags that
    ! can alter the flow of the model in a control structure. 
    control%ok_land_cover_change = control_in%ok_land_cover_change
    ! The do_land_use flag is used in slow_proc.f90 but it looks like it should not
    ! have been defined in this structure but simply as a local logical because
    ! it is used to update the land use maps but it does not affect the flow of the 
    ! code by itself as it is NOT set by the user
    ! +++++++++++
   IF (long_print) WRITE (numout,*) ' sechiba_init done '

  END SUBROUTINE sechiba_init
  

!! ==============================================================================================================================\n
!! SUBROUTINE 	: sechiba_clear
!!
!>\BRIEF        Deallocate memory of sechiba's variables
!!
!! DESCRIPTION  : None
!!
!! RECENT CHANGE(S): None
!!
!! MAIN OUTPUT VARIABLE(S): None 
!!
!! REFERENCE(S)	: None
!!
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================ 

  SUBROUTINE sechiba_clear (forcing_name,cforcing_name)

    CHARACTER(LEN=100), INTENT(in)           :: forcing_name       !! Name of forcing file (unitless)
    CHARACTER(LEN=100), INTENT(in)           :: cforcing_name      !! Name of forcing file with carbon related variables (unitless)
!_ ================================================================================================================================
    
!! 1. Initialize first run

    l_first_sechiba=.TRUE.

!! 2. Deallocate dynamic variables of sechiba

    IF ( ALLOCATED (indexveg)) DEALLOCATE (indexveg)
    IF ( ALLOCATED (indexlai)) DEALLOCATE (indexlai)
    IF ( ALLOCATED (indexsoil)) DEALLOCATE (indexsoil)
    IF ( ALLOCATED (indexnobio)) DEALLOCATE (indexnobio)
    IF ( ALLOCATED (indexsnow)) DEALLOCATE (indexsnow) 
    IF ( ALLOCATED (indexgrnd)) DEALLOCATE (indexgrnd)
    IF ( ALLOCATED (indexlayer)) DEALLOCATE (indexlayer)
    IF ( ALLOCATED (indexnbdl)) DEALLOCATE (indexnbdl) 
    IF ( ALLOCATED (indexalb)) DEALLOCATE (indexalb)
    IF ( ALLOCATED (indexcan)) DEALLOCATE (indexcan)
    IF ( ALLOCATED (flood_res)) DEALLOCATE (flood_res)
    IF ( ALLOCATED (flood_frac)) DEALLOCATE (flood_frac)
    IF ( ALLOCATED (snow)) DEALLOCATE (snow)
    IF ( ALLOCATED (snow_age)) DEALLOCATE (snow_age)
    IF ( ALLOCATED (drysoil_frac)) DEALLOCATE (drysoil_frac)
    IF ( ALLOCATED (rsol)) DEALLOCATE (rsol)
    IF ( ALLOCATED (evap_bare_lim)) DEALLOCATE (evap_bare_lim)
    IF ( ALLOCATED (evapot)) DEALLOCATE (evapot)
    IF ( ALLOCATED (evapot_corr)) DEALLOCATE (evapot_corr)
    IF ( ALLOCATED (humrel)) DEALLOCATE (humrel)
    IF ( ALLOCATED (vegstress)) DEALLOCATE (vegstress)
    IF ( ALLOCATED (soiltile)) DEALLOCATE (soiltile)
    IF ( ALLOCATED (njsc)) DEALLOCATE (njsc)
    IF ( ALLOCATED (reinf_slope)) DEALLOCATE (reinf_slope)
    IF ( ALLOCATED (vbeta1)) DEALLOCATE (vbeta1)
    IF ( ALLOCATED (vbeta4)) DEALLOCATE (vbeta4)
    IF ( ALLOCATED (vbeta5)) DEALLOCATE (vbeta5)
    IF ( ALLOCATED (soilcap)) DEALLOCATE (soilcap)
    IF ( ALLOCATED (soilflx)) DEALLOCATE (soilflx)
    IF ( ALLOCATED (temp_sol)) DEALLOCATE (temp_sol)
    IF ( ALLOCATED (qsurf)) DEALLOCATE (qsurf)
    IF ( ALLOCATED (qsintveg)) DEALLOCATE (qsintveg)
    IF ( ALLOCATED (vbeta2))  DEALLOCATE (vbeta2)
    IF ( ALLOCATED (vbeta3)) DEALLOCATE (vbeta3)
    IF ( ALLOCATED (vbeta3pot)) DEALLOCATE (vbeta3pot)
    IF ( ALLOCATED (gsmean)) DEALLOCATE (gsmean)
!    IF ( ALLOCATED (vbetaco2)) DEALLOCATE (vbetaco2)
    IF ( ALLOCATED (cimean)) DEALLOCATE (cimean)
    IF ( ALLOCATED (gpp)) DEALLOCATE (gpp)
    IF ( ALLOCATED (temp_growth)) DEALLOCATE (temp_growth)
    IF ( ALLOCATED (veget)) DEALLOCATE (veget)
    IF ( ALLOCATED (veget_max)) DEALLOCATE (veget_max)
    IF ( ALLOCATED (lai)) DEALLOCATE (lai)
    IF ( ALLOCATED (laieff_fit)) DEALLOCATE (laieff_fit)
    IF ( ALLOCATED (frac_age)) DEALLOCATE (frac_age)
    IF ( ALLOCATED (height)) DEALLOCATE (height)
    IF ( ALLOCATED (roughheight)) DEALLOCATE (roughheight)
    IF ( ALLOCATED (z0_veg)) DEALLOCATE (z0_veg)
    IF ( ALLOCATED (frac_nobio)) DEALLOCATE (frac_nobio)
    IF ( ALLOCATED (snow_nobio)) DEALLOCATE (snow_nobio)
    IF ( ALLOCATED (snow_nobio_age)) DEALLOCATE (snow_nobio_age)
    IF ( ALLOCATED (assim_param)) DEALLOCATE (assim_param)
    IF ( ALLOCATED (vevapflo)) DEALLOCATE (vevapflo)
    IF ( ALLOCATED (vevapsno)) DEALLOCATE (vevapsno)
    IF ( ALLOCATED (vevapnu)) DEALLOCATE (vevapnu)
    IF ( ALLOCATED (t2mdiag)) DEALLOCATE (t2mdiag)
    IF ( ALLOCATED (totfrac_nobio)) DEALLOCATE (totfrac_nobio)
    IF ( ALLOCATED (floodout)) DEALLOCATE (floodout)
    IF ( ALLOCATED (runoff)) DEALLOCATE (runoff)
    IF ( ALLOCATED (drainage)) DEALLOCATE (drainage)
    IF ( ALLOCATED (reinfiltration)) DEALLOCATE (reinfiltration)
    IF ( ALLOCATED (irrigation)) DEALLOCATE (irrigation)
    IF ( ALLOCATED (tot_melt)) DEALLOCATE (tot_melt)
    IF ( ALLOCATED (valpha)) DEALLOCATE (valpha)
    IF ( ALLOCATED (vbeta)) DEALLOCATE (vbeta)
    IF ( ALLOCATED (fusion)) DEALLOCATE (fusion)
    IF ( ALLOCATED (rau)) DEALLOCATE (rau)
    IF ( ALLOCATED (deadleaf_cover)) DEALLOCATE (deadleaf_cover)
    IF ( ALLOCATED (stempdiag)) DEALLOCATE (stempdiag)
    IF ( ALLOCATED (co2_flux)) DEALLOCATE (co2_flux)
    IF ( ALLOCATED (shumdiag)) DEALLOCATE (shumdiag)
    IF ( ALLOCATED (shumdiag_perma)) DEALLOCATE (shumdiag_perma)
    IF ( ALLOCATED (litterhumdiag)) DEALLOCATE (litterhumdiag)
    IF ( ALLOCATED (ptnlev1)) DEALLOCATE (ptnlev1)
    IF ( ALLOCATED (k_litt)) DEALLOCATE (k_litt)
    IF ( ALLOCATED (vevapwet)) DEALLOCATE (vevapwet)
    IF ( ALLOCATED (transpir)) DEALLOCATE (transpir)
    IF ( ALLOCATED (stressed)) DEALLOCATE (stressed)
    IF ( ALLOCATED (unstressed)) DEALLOCATE (unstressed)
!!$    IF ( ALLOCATED (vir_transpir_mod)) DEALLOCATE (vir_transpir_mod)
    IF ( ALLOCATED (transpir_supply)) DEALLOCATE (transpir_supply)
    IF ( ALLOCATED (vir_transpir_supply)) DEALLOCATE (vir_transpir_supply)
    IF ( ALLOCATED (transpot)) DEALLOCATE (transpot)
    IF ( ALLOCATED (qsintmax)) DEALLOCATE (qsintmax)
    IF ( ALLOCATED (rveget)) DEALLOCATE (rveget)
    IF ( ALLOCATED (rstruct)) DEALLOCATE (rstruct)
    IF ( ALLOCATED (snowrho)) DEALLOCATE (snowrho) 
    IF ( ALLOCATED (snowgrain)) DEALLOCATE (snowgrain) 
    IF ( ALLOCATED (snowtemp)) DEALLOCATE (snowtemp) 
    IF ( ALLOCATED (soiltemp)) DEALLOCATE (soiltemp) 
    IF ( ALLOCATED (snowdz)) DEALLOCATE (snowdz) 
    IF ( ALLOCATED (snowdz_o)) DEALLOCATE (snowdz_o) 
    IF ( ALLOCATED (snowliq)) DEALLOCATE (snowliq) 
    IF ( ALLOCATED (snowheat)) DEALLOCATE (snowheat) 
    IF ( ALLOCATED (grndflux)) DEALLOCATE (grndflux) 
    IF ( ALLOCATED (radxs)) DEALLOCATE (radxs) 
    IF ( ALLOCATED (radsink)) DEALLOCATE (radsink) 
    IF ( ALLOCATED (sfcfrz)) DEALLOCATE (sfcfrz) 
    IF ( ALLOCATED (gtemp)) DEALLOCATE (gtemp) 
    IF ( ALLOCATED (soilflxresid)) DEALLOCATE (soilflxresid) 
    IF ( ALLOCATED (gpkappa)) DEALLOCATE (gpkappa) 
    IF ( ALLOCATED (gthick)) DEALLOCATE (gthick) 
    IF ( ALLOCATED (pgflux)) DEALLOCATE (pgflux) 
    IF ( ALLOCATED (albedo_undersnow)) DEALLOCATE (albedo_undersnow) 
    IF ( ALLOCATED (osfcmelt)) DEALLOCATE (osfcmelt) 
    IF ( ALLOCATED (pkappa_snow)) DEALLOCATE (pkappa_snow) 
    IF ( ALLOCATED (warnings)) DEALLOCATE (warnings)
    IF ( ALLOCATED (lai_split)) DEALLOCATE (lai_split)
    IF (ALLOCATED(lai_per_level)) DEALLOCATE(lai_per_level)
    IF (ALLOCATED(circ_class_n)) DEALLOCATE(circ_class_n)
    IF (ALLOCATED(circ_class_biomass)) DEALLOCATE(circ_class_biomass)
    IF (ALLOCATED(biomass)) DEALLOCATE(biomass)
    IF (ALLOCATED(ind)) DEALLOCATE(ind) 
    IF (ALLOCATED(Isotrop_Abs_Tot_p)) DEALLOCATE(Isotrop_Abs_Tot_p)
    IF (ALLOCATED(frac_snow_veg)) DEALLOCATE(frac_snow_veg)
    IF (ALLOCATED(frac_snow_nobio)) DEALLOCATE(frac_snow_nobio)
    IF (ALLOCATED(frac_snow_pix)) DEALLOCATE(frac_snow_pix)
    IF (ALLOCATED(sum_veget_diff)) DEALLOCATE(sum_veget_diff)
    IF ( ALLOCATED (transpir_mod)) DEALLOCATE (transpir_mod)

    IF (ALLOCATED(flux_h_grid)) DEALLOCATE(flux_h_grid)
    IF (ALLOCATED(flux_le_grid)) DEALLOCATE(flux_le_grid)
    IF (ALLOCATED(u_speed_grid)) DEALLOCATE(u_speed_grid)
    IF (ALLOCATED(t_a_next_grid)) DEALLOCATE(t_a_next_grid)
    IF (ALLOCATED(q_a_next_grid)) DEALLOCATE(q_a_next_grid)
    IF (ALLOCATED(temp_atmos_pres_grid)) DEALLOCATE(temp_atmos_pres_grid)
    IF (ALLOCATED(q_atmos_pres_grid)) DEALLOCATE(q_atmos_pres_grid)
    IF (ALLOCATED(temp_leaf_pres_grid)) DEALLOCATE(temp_leaf_pres_grid)
    IF (ALLOCATED(swc)) DEALLOCATE(swc)
    IF (ALLOCATED(albedo_pft)) DEALLOCATE(albedo_pft)

    IF ( ALLOCATED (jnlevels_loc)) DEALLOCATE (jnlevels_loc)   ! JR
    IF (ALLOCATED(u_speed)) DEALLOCATE(u_speed)
    IF (ALLOCATED(max_height_store)) DEALLOCATE(max_height_store)
    IF (ALLOCATED(h_array_out)) DEALLOCATE(h_array_out)
    IF (ALLOCATED(z_array_out)) DEALLOCATE(z_array_out)
    IF (ALLOCATED(t2m_month_out)) DEALLOCATE(t2m_month_out)
    IF (ALLOCATED(profile_vbeta3)) DEALLOCATE(profile_vbeta3)
    IF (ALLOCATED(profile_rveget)) DEALLOCATE(profile_rveget)

    IF ( ALLOCATED (delta_c13_assim)) DEALLOCATE (delta_c13_assim)
    IF ( ALLOCATED (leaf_ci_out)) DEALLOCATE (leaf_ci_out)

!! 3. Clear all allocated memory

    CALL pft_parameters_clear
    CALL slowproc_clear 
    CALL diffuco_clear 
    CALL enerbil_clear  
    IF ( hydrol_cwrr ) THEN
       CALL hydrol_clear 
    ELSE
       CALL hydrolc_clear  
    ENDIF
    CALL condveg_clear 
    CALL thermosoil_clear
    CALL routing_clear

  END SUBROUTINE sechiba_clear


!! ==============================================================================================================================\n
!! SUBROUTINE 	: sechiba_var_init
!!
!>\BRIEF        Calculate air density as a function of air temperature and 
!! pressure for each terrestrial pixel.
!! 
!! RECENT CHANGE(S): None
!! 
!! MAIN OUTPUT VARIABLE(S): air density (::rau, kg m^{-3}).
!! 
!! REFERENCE(S)	: None
!! 
!! FLOWCHART    : None
!! \n
!_ ================================================================================================================================

  SUBROUTINE sechiba_var_init (kjpindex, rau, pb, temp_air) 

!! 0.1 Input variables

    INTEGER(i_std), INTENT (in)                    :: kjpindex        !! Domain size - terrestrial pixels only (unitless)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)  :: pb              !! Lowest level pressure (hPa)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)  :: temp_air        !! Air temperature (K)
    
!! 0.2 Output variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (out) :: rau             !! Air density @tex $(kg m^{-3})$ @endtex

!! 0.3 Modified variables

!! 0.4 Local variables

    INTEGER(i_std)                                 :: ji              !! Indices (unitless)
!_ ================================================================================================================================
    
!! 1. Calculate intial air density (::rau)
   
    DO ji = 1,kjpindex
       rau(ji) = pa_par_hpa * pb(ji) / (cte_molr*temp_air(ji))
    END DO

    IF (long_print) WRITE (numout,*) ' sechiba_var_init done '

  END SUBROUTINE sechiba_var_init


!! ==============================================================================================================================\n
!! SUBROUTINE 	: sechiba_end
!!
!>\BRIEF        Swap old for newly calculated soil temperature.
!! 
!! RECENT CHANGE(S): None
!! 
!! MAIN OUTPUT VARIABLE(S): soil temperature (::temp_sol; K)
!! 
!! REFERENCE(S)	: None
!! 
!! FLOWCHART    : None
!! \n
!! ================================================================================================================================ 

  SUBROUTINE sechiba_end (kjpindex, dtradia, temp_sol_new, snowtemp, snowdz, & 
       temp_sol)

  !! 0.1 Input variables

    INTEGER(i_std), INTENT (in)                       :: kjpindex           !! Domain size - terrestrial pixels only (unitless)
    REAL(r_std),INTENT (in)                           :: dtradia            !! Time step (seconds)
    REAL(r_std),DIMENSION (kjpindex), INTENT (in)     :: temp_sol_new       !! New soil temperature (K)
    REAL(r_std),DIMENSION (kjpindex,nsnow),INTENT(in) :: snowtemp           !! Snow temperature (K) 
    REAL(r_std),DIMENSION (kjpindex,nsnow),INTENT(in) :: snowdz             !! Snow layer thickness (m)

  !! 0.2 Output variables

    REAL(r_std),DIMENSION (kjpindex), INTENT (out)    :: temp_sol           !! Soil temperature (K)

  !! 0.3 Modified variables

  !! 0.4 Local variables

    INTEGER(i_std) :: ji 

!_ ================================================================================================================================
    
!! 1. Swap temperature

    IF (ok_explicitsnow) THEN 
       DO ji=1,kjpindex 
          ! When the snow does not exist on the ground, the old surface temperature 
          ! should be taken from the previous snow surface temperature 
          IF (SUM(snowdz(ji,:)) .GT. 0.0) THEN 
             temp_sol(ji) = snowtemp(ji,1) 
          ELSE 
             temp_sol(ji) = temp_sol_new(ji) 
          END IF 
       END DO 

    ELSE 
       temp_sol(:) = temp_sol_new(:) 
    END IF 

    IF (long_print) WRITE (numout,*) ' sechiba_end done '

  END SUBROUTINE sechiba_end

END MODULE sechiba
